/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003, 2004    (see file CONTACTS for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#ifndef __BIASERROR_HH__
#define __BIASERROR_HH__
#include "bias_config.h" 


#include <iostream>

#define ERROR_STREAM std::cerr 

/// macro for error handling
#define BIASERR(arg)\
      ERROR_STREAM<<std::flush<<"Error in "<<__FILE__<<":"<<__LINE__\
                  <<"   :\n\t"<<arg<<std::endl<<std::flush;

#define BIASWARN(arg)\
      ERROR_STREAM<<std::flush<<"Warning in "<<__FILE__<<":"<<__LINE__\
                  <<"   :\n\t"<<arg<<std::endl<<std::flush;



// this does not work, it is just a template
/* #define BIAS_MUTEX_LOCK(arg)\
       cerr <<__FILE__<<":"<<__LINE__<<" attempting to lock mutex .. "<<#arg;\
       cerr.flush();\
       pthread_mutex_lock(arg);\
       cerr <<" .. got lock\n";
 #define BIAS_MUTEX_UNLOCK(arg)\
       cerr <<__FILE__<<":"<<__LINE__<<" unlocking mutex .. "<<#arg;\
       cerr.flush();\
       pthread_mutex_unlock(arg);\
       cerr <<" .. unlocked\n";
*/
#ifndef WIN32 

#define BIAS_MUTEX_LOCK(arg) pthread_mutex_lock(arg);
#define BIAS_MUTEX_UNLOCK(arg) pthread_mutex_unlock(arg);

#else 

#define BIAS_MUTEX_LOCK(arg) 
#define BIAS_MUTEX_UNLOCK(arg)

#endif


#define BIASNEW(arg1,arg2)\
      try {arg1= new arg2;} catch (std::bad_alloc){ BIASERR("new failed");};

/** 
    @brief Macros to be used instead of assert()
    @author Jan Woetzel
    @status tested (09/2003) **/

#include <assert.h>
// assertions are also switched of if NDEBUG is defined 


#ifdef BIAS_DEBUG
// do assertion
#    define BIASASSERT(cond) \
if (!(cond)) {std::cout<<std::flush; std::cerr<<std::flush<<__FILE__<<", "<<__LINE__<<" : "<<std::endl<<std::flush;}; \
assert(cond);

#    define BIAS_CHECK_NAN(x) if (x!=x) ERROR_STREAM<<"NaN fever for "<<\
             __STRING(x)<<" file: "<<__FILE__<<" line: "<<__LINE__<<std::endl;

#  else
// do nothing
#    define BIASASSERT(cond)
#    define BIAS_CHECK_NAN(x) {}
#  endif


/**
   @brief Macro to be used instead of exit()
   @autor Jan Woetzel
   @status trivial but untested 09/2003 **/
#define BIASEXIT(mesg) \
std::cerr<<std::flush<<std::endl<<"exit request in "<<__FILE__<<","<<__LINE__<<" : "<<std::endl<<mesg<<std::endl; exit(-1);



#ifdef VISUALCPP
  #define THROW_LIST(E) /* throws: E */
#else

  #ifdef BIAS_NOEXCEPTIONS
    #define THROW_LIST(E) /* E */
  #else
    #define THROW_LIST(E) throw (E)
  #endif

#endif


#ifdef BIAS_DEBUG

#ifdef BIAS_NOEXCEPTIONS
#define THROW(E) BIASGDOUT(0, "Exception thrown in " << __FILE__ \
                           << "at line " << __LINE__)
#else
#define THROW(E) throw E; \
    BIASGDOUT(0, "Exception thrown in " << __FILE__ << "at line " << __LINE__)
#endif
#else
#ifdef BIAS_NOEXCEPTIONS
#  define THROW(E) /* E */
#else
#  define THROW(E) throw E
#endif
								     
#endif // ifdef BASE_DEBUG
								     
								     
// macro to dislay the complete function name a line of code belongs to.(see MSDN or g++ docu) 
// work with g++ 3.3 and MS Visual Studo 7.1 
// Jan Woetzel  12/2004
#ifdef WIN32								     
#  define FUNCNAME  __FUNCTION__
#else // WIN32
#  define FUNCNAME  __PRETTY_FUNCTION__
#endif // WIN32


// JW
#define CALLINFO std::cout<<"called "<<__FILE__<<", "<<__LINE__<<": "<<FUNCNAME<<std::endl<<std::flush;



#define DEF_EXCEPTION(theClass, theText) \
class DLLEXPORTAPI theClass : public Exception { theClass() : Exception(theText) {} };

#endif // __BIASERROR_HH__
