/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003, 2004    (see file CONTACTS for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#ifndef __KMatrix_hh__
#define __KMatrix_hh__
#include "bias_config.h" 

#include <Base/Math/Matrix3x3.hh>

#define KMATRIX_TYPE double

namespace BIAS{
  /** @class KMatrix
      @ingroup g_geometry
      @brief
      K describes the mapping from world coordinates (wcs) to pixel 
      coordinates (pcs).
      To remember: 
      (1) (R' | -R' C) projects a homgenous 3D (scene) point from wcs to 
                       2d ics.
      (2) K            transfroms a 2d scene point from ics to pcs 
                       (pixel coordinates) (which is mainly shift of origin 
                        + scale).
      @author Jan Woetzel
  */
  class DLLEXPORTAPI KMatrix : public Matrix3x3<KMATRIX_TYPE>
  {
  public:
    KMatrix() : Matrix3x3<KMATRIX_TYPE>() {};

    KMatrix(const Matrix3x3<KMATRIX_TYPE>& A) : Matrix3x3<KMATRIX_TYPE>(A) {};

    ~KMatrix() {};

    /** returns analyticaly inverted matrix */
    KMatrix Invert() const;

  };

} // namespace 


#endif // __KMatrix_hh__
