/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003, 2004    (see file CONTACTS for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include "ImageDraw.hh"

#include "Bresenham.hh"
#include "BresenhamCircle.hh"

using namespace BIAS;
using namespace std;

template <class StorageType>
StorageType ImageDraw<StorageType>::GetContrastValue(StorageType t)
{
  BIASERR("not specialized from StorageType"); 
  return (StorageType)0;
}

template <>
unsigned char ImageDraw<unsigned char>::GetContrastValue(unsigned char t)
{
  return ((int)t>128)?(unsigned char)0:(unsigned char)255;
}

template <>
float ImageDraw<float>::GetContrastValue(float t)
{
  return float((t>0.0)?-1.0:1.0);
}


/////////
// rectangles

template <class StorageType>
int ImageDraw<StorageType>::RectangleCorners(Image<StorageType>& im, 
                                             unsigned int minx, 
                                             unsigned int miny,
                                             unsigned int maxx, 
                                             unsigned int maxy,
                                             StorageType Value[])
{
  BIASASSERT(minx<im.GetWidth());
  BIASASSERT(miny<im.GetHeight());
  BIASASSERT(maxx<im.GetWidth());
  BIASASSERT(maxy<im.GetHeight());
   
  register StorageType **ida = im.GetImageDataArray();
  register unsigned int xp1, xp2;
  for (register unsigned int x = minx; x <= maxx; x++){
    xp1 = x * im.GetChannelCount();
    for (register unsigned int c=0; c< im.GetChannelCount(); c++)
      ida[miny][xp1+c]=ida[maxy][xp1+c]=Value[c];
  }
  // left and right lines
  xp1=minx*im.GetChannelCount();
  xp2=maxx*im.GetChannelCount();
  for (register unsigned int y = miny; y <= maxy; y++){
    for (register unsigned int c=0; c< im.GetChannelCount(); c++)
      ida[y][xp1+c]=ida[y][xp2+c]=Value[c];
  }
  return 0;
}


template <class StorageType>
int ImageDraw<StorageType>::RectangleCorners(Image<StorageType>& im, 
                                      unsigned int minx, 
                                      unsigned int miny,
                                      unsigned int maxx, 
                                      unsigned int maxy,
                                      StorageType value)
{
  StorageType *val = new StorageType[im.GetChannelCount()];
  for (register unsigned int i=0; i<im.GetChannelCount(); i++)
    val[i]=value;
  return RectangleCorners(im, minx, miny, maxx, maxy, val);
}


template <class StorageType>
int ImageDraw<StorageType>::RectangleCorners(Image<StorageType>& im, 
                                      unsigned int minx, 
                                      unsigned int miny,
                                      unsigned int maxx, 
                                      unsigned int maxy)
{
  register StorageType **ida = im.GetImageDataArray();
  register unsigned int xp1, xp2;
  for (register unsigned int x = minx; x <= maxx; x++){
    xp1 = x * im.GetChannelCount();
    for (register unsigned int c=0; c< im.GetChannelCount(); c++){
      ida[miny][xp1+c]=GetContrastValue(ida[miny][xp1+c]);
      ida[maxy][xp1+c]=GetContrastValue(ida[maxy][xp1+c]);
    }
  }
  // left and right lines
  xp1=minx*im.GetChannelCount();
  xp2=maxx*im.GetChannelCount();
  for (register unsigned int y = miny; y <= maxy; y++){
    for (register unsigned int c=0; c< im.GetChannelCount(); c++){
      ida[y][xp1+c]=GetContrastValue(ida[y][xp1+c]);
      ida[y][xp2+c]=GetContrastValue(ida[y][xp2+c]);
    }
  }
  return 0;
}

template <class StorageType>
int ImageDraw<StorageType>::RectangleCornersGrey(Image<StorageType>& im,
                                          unsigned int minx, 
                                          unsigned int miny,
                                          unsigned int maxx, 
                                          unsigned int maxy,
                                          StorageType GreyValue)
{
  register StorageType *ImageDataPointer, *ImageDataPointer2; 
  register unsigned int Width = im.GetWidth();
  // horizontal lines lines
  ImageDataPointer = im.GetImageData() + 
    im.GetPixelPosition(minx,miny);
  ImageDataPointer2 = im.GetImageData() + 
    im.GetPixelPosition(minx,maxy);
  for (register unsigned int i = 0; i <= maxx - minx; i++){
    *ImageDataPointer = GreyValue;
    *ImageDataPointer2 = GreyValue;
    ImageDataPointer++;
    ImageDataPointer2++;
  }
  // left and right lines
  ImageDataPointer = im.GetImageData() + 
    im.GetPixelPosition(minx,miny);
  ImageDataPointer2 = im.GetImageData() + 
    im.GetPixelPosition(maxx,miny);
  // top and bottom lines
  for (register unsigned int i = 0; i <= maxy - miny; i++){
    *ImageDataPointer = GreyValue;
    *ImageDataPointer2 = GreyValue;
    ImageDataPointer += Width;
    ImageDataPointer2 += Width;
  }
  return 0;
}


template <class StorageType>
int ImageDraw<StorageType>::RectangleCornersGreyFill(Image<StorageType>& im,
                                                     unsigned int minx, 
                                                     unsigned int miny,
                                                     unsigned int maxx, 
                                                     unsigned int maxy,
                                                     StorageType GreyValue)
{
  register StorageType *ImageDataPointer;
  
  for (unsigned int row=miny;row<=maxy;row++) {
    ImageDataPointer = im.GetImageData() + 
      im.GetPixelPosition(minx,row);
    for (register unsigned int i = 0; i <= maxx - minx; i++){
      *ImageDataPointer = GreyValue;
      ImageDataPointer++;
    }
  }
  return 0;
}


template <class StorageType>
int ImageDraw<StorageType>::RectangleCenter(Image<StorageType>& im, 
                                      unsigned int X, unsigned int Y, 
                                      unsigned int Size, StorageType Value[])
{
#ifdef BIAS_DEBUG
  if (Size > (unsigned int)im.GetWidth() || 
      Size > (unsigned int)im.GetHeight())
    BIASERR("wrong Size "<<Size);
#endif
  unsigned int Halfsize = Size >> 1;
  if (im.GetChannelCount()==1){
    return RectangleCornersGrey(im, (X>Halfsize)?X-Halfsize:0, 
                             (Y>Halfsize)?Y-Halfsize:0, 
                             (X<im.GetWidth()-Halfsize)?
                             (X+Halfsize):(im.GetWidth()-1), 
                             (Y<im.GetHeight()-Halfsize)?
                             (Y+Halfsize):(im.GetHeight()-1),
                             Value[0]); 
  } else {
    return RectangleCorners(im, (X>Halfsize)?X-Halfsize:0, 
                     (Y>Halfsize)?Y-Halfsize:0, 
                     (X<im.GetWidth()-Halfsize)?
                     (X+Halfsize):(im.GetWidth()-1), 
                     (Y<im.GetHeight()-Halfsize)?
                     (Y+Halfsize):(im.GetHeight()-1),
                     Value);
  }
}


template <class StorageType>
int ImageDraw<StorageType>::RectangleCenterGrey(Image<StorageType>& im, 
                                          unsigned int X, unsigned int Y, 
                                          unsigned int Size, StorageType Value)
{
#ifdef BIAS_DEBUG
  if (Size > (unsigned int)im.GetWidth() || 
      Size > (unsigned int)im.GetHeight())
    BIASERR("wrong Size "<<Size);
#endif
  unsigned int Halfsize = Size >> 1;
  return RectangleCornersGrey(im, (X>Halfsize)?X-Halfsize:0, 
                           (Y>Halfsize)?Y-Halfsize:0, 
                           (X<im.GetWidth()-Halfsize)?
                           (X+Halfsize):(im.GetWidth()-1), 
                           (Y<im.GetHeight()-Halfsize)?
                           (Y+Halfsize):(im.GetHeight()-1),
                           Value); 
}

template <class StorageType>
int ImageDraw<StorageType>::RectangleCenterGreyFill(Image<StorageType>& im, 
                                                    unsigned int X, 
                                                    unsigned int Y, 
                                                    unsigned int Size, 
                                                    StorageType Value)
{
#ifdef BIAS_DEBUG
  if (Size > (unsigned int)im.GetWidth() || 
      Size > (unsigned int)im.GetHeight())
    BIASERR("wrong Size "<<Size);
#endif
  unsigned int Halfsize = Size >> 1;
  return RectangleCornersGreyFill(im, (X>Halfsize)?X-Halfsize:0, 
                           (Y>Halfsize)?Y-Halfsize:0, 
                           (X<im.GetWidth()-Halfsize)?
                           (X+Halfsize):(im.GetWidth()-1), 
                           (Y<im.GetHeight()-Halfsize)?
                           (Y+Halfsize):(im.GetHeight()-1),
                           Value); 
}



template <class StorageType>
int ImageDraw<StorageType>::RectangleCenter(Image<StorageType>& im, unsigned int x, 
                                      unsigned int y, unsigned int size)
{
  unsigned int Halfsize = size >> 1;
  return RectangleCorners(im, (x>Halfsize)?x-Halfsize:0, 
                   (y>Halfsize)?y-Halfsize:0, 
                   (x<im.GetWidth()-Halfsize)?(x+Halfsize):(im.GetWidth()-1),
                   (y<im.GetHeight()-Halfsize)?
                   (y+Halfsize):(im.GetHeight()-1));
}

template <class StorageType>
int ImageDraw<StorageType>::RectangleCenter(Image<StorageType>& im, 
                                      unsigned int coo[2], 
                                      unsigned int size)
{ return RectangleCenter(im, coo[0], coo[1], size); }

/////////////////////////////////////////////////////////////////////////
// lines 
/////////////////////////////////////////////////////////////////////////


template <class StorageType>
int ImageDraw<StorageType>::Line(Image<StorageType>& im,
                                 const unsigned int start[2],
                                 const unsigned int end[2], 
                                 StorageType value[])
{
#ifdef BIAS_DEBUG
  if (start[0]>=im.GetWidth() ||
      start[1]>=im.GetHeight() ||
      end[0]>=im.GetWidth() ||
      end[1]>=im.GetHeight()){
    BIASASSERT(false);
    return -1;
  }
#endif
  unsigned int next[2];
  Bresenham bres(start, end);

  next[0]=start[0];
  next[1]=start[1];
  
  StorageType **ida=im.GetImageDataArray();
  do {
    if (next[0]<0 || next[0]>=im.GetWidth() ||
        next[1]<0 || next[1]>=im.GetHeight())
      break;
    for (unsigned int c=0; c<im.GetChannelCount(); c++)
      ida[next[1]][next[0]*im.GetChannelCount()+c]=value[c];
  } while (bres.GetNext(next));
  return 0;
}

template <class StorageType>
int ImageDraw<StorageType>::Line(Image<StorageType>& im,
                                 const unsigned int start[2],
                                 const unsigned int end[2], StorageType value)
{
#ifdef BIAS_DEBUG
  if (start[0]<0 || start[0]>=im.GetWidth() ||
      start[1]<0 || start[1]>=im.GetHeight() ||
      end[0]<0 || end[0]>=im.GetWidth() ||
      end[1]<0 || end[1]>=im.GetHeight()){
    BIASERR("coordinates out of bounds "<<start[0]<<" "<<start[1]<<" "
            <<end[0]<<" "<<end[1]);
    return -1;
  }
#endif
  unsigned int next[2];
  Bresenham bres(start, end);

  next[0]=start[0];
  next[1]=start[1];
  
  StorageType **ida=im.GetImageDataArray();
  do {
    for (unsigned int c=0; c<im.GetChannelCount(); c++)
      ida[next[1]][next[0]*im.GetChannelCount()+c]=value;
  } while (bres.GetNext(next));
  return 0;
}

template <class StorageType>
int ImageDraw<StorageType>::LineGrey(Image<StorageType>& im,
                                     const unsigned int start[2],
                                     const unsigned int end[2], 
                                     StorageType value)
{
#ifdef BIAS_DEBUG
  if (start[0]<0 || start[0]>=im.GetWidth() ||
      start[1]<0 || start[1]>=im.GetHeight() ||
      end[0]<0 || end[0]>=im.GetWidth() ||
      end[1]<0 || end[1]>=im.GetHeight()){
    BIASERR("coordinates out of bounds");
    return -1;
  }
#endif
  unsigned int next[2];
  Bresenham bres(start, end);

  next[0]=start[0];
  next[1]=start[1];
  
  StorageType **ida=im.GetImageDataArray();
  do {
      ida[next[1]][next[0]]=value;
  } while (bres.GetNext(next));
  return 0;
}


template <class StorageType>
int ImageDraw<StorageType>::Line(Image<StorageType>& im,
                                 const unsigned int start[2],
                                 const unsigned int end[2])
{
#ifdef BIAS_DEBUG
  if (start[0]<0 || start[0]>=im.GetWidth() ||
      start[1]<0 || start[1]>=im.GetHeight() ||
      end[0]<0 || end[0]>=im.GetWidth() ||
      end[1]<0 || end[1]>=im.GetHeight()){
    BIASERR("coordinates out of bounds: "<<start[0]<<";"<<start[1]
            <<" to "<<end[0]<<";"<<end[1]);
    return -1;
  }
#endif
  unsigned int next[2];
  Bresenham bres(start, end);

  next[0]=start[0];
  next[1]=start[1];
  
  StorageType **ida=im.GetImageDataArray();
  do {
    for (unsigned int c=0; c<im.GetChannelCount(); c++)
      ida[next[1]][next[0]*im.GetChannelCount()+c]=
        GetContrastValue(ida[next[1]][next[0]*im.GetChannelCount()+c]);
  } while (bres.GetNext(next));
  return 0;
}


template <class StorageType>
int ImageDraw<StorageType>::LineGrey(Image<StorageType>& im,
                                     const unsigned int start[2],
                                     const unsigned int end[2])
{
#ifdef BIAS_DEBUG
  if (start[0]<0 || start[0]>=im.GetWidth() ||
      start[1]<0 || start[1]>=im.GetHeight() ||
      end[0]<0 || end[0]>=im.GetWidth() ||
      end[1]<0 || end[1]>=im.GetHeight()){
    BIASERR("coordinates out of bounds");
    return -1;
  }
  if (im.GetChannelCount() !=1){
    BIASERR("only for one channel images");
    return -2;
  }
#endif
  unsigned int next[2];
  Bresenham bres(start, end);

  next[0]=start[0];
  next[1]=start[1];
  
  StorageType **ida=im.GetImageDataArray();
  do {
      ida[next[1]][next[0]]=GetContrastValue(ida[next[1]][next[0]]);
  } while (bres.GetNext(next));
  return 0;
}



template <class StorageType>
int ImageDraw<StorageType>::Line(Image<StorageType>& im, 
                                 const unsigned int StartX, 
                                 const unsigned int StartY, 
                                 const unsigned int EndX, 
                                 const unsigned int EndY, 
                                 StorageType Value[])
{
  unsigned int start[]={StartX, StartY}, end[]={EndX, EndY};
  return Line(im, start, end, Value); 
}


template <class StorageType>
int ImageDraw<StorageType>::Line(Image<StorageType>& im, 
                                 const unsigned int StartX, 
                                 const unsigned int StartY, 
                                 const unsigned int EndX, 
                                 const unsigned int EndY, 
                                 StorageType Value)
{
  unsigned int start[]={StartX, StartY}, end[]={EndX, EndY};
  return Line(im, start, end, Value); 
}

template <class StorageType>
int ImageDraw<StorageType>::Line(Image<StorageType>& im, 
                                 const unsigned int StartX, 
                                 const unsigned int StartY, 
                                 const unsigned int EndX, 
                                 const unsigned int EndY)
{
  unsigned int start[]={StartX, StartY}, end[]={EndX, EndY};
  return Line(im, start, end); 
}



template <class StorageType>
int ImageDraw<StorageType>::LineGrey(Image<StorageType>& im,
                                     const unsigned int StartX, 
                                     const unsigned int StartY,
                                     const unsigned int EndX, 
                                     const unsigned int EndY, 
                                     StorageType Value)
{
  unsigned int start[]={StartX, StartY}, end[]={EndX, EndY};
  return LineGrey(im, start, end, Value); 
}

// template <class StorageType>
// int ImageDraw<StorageType>::LineGrey(Image<StorageType>& im,
//                                      const unsigned int StartX, 
//                                      const unsigned int StartY,
//                                      const unsigned int EndX, 
//                                      const unsigned int EndY, 
//                                      StorageType Value)
// {
// #ifdef BIAS_DEBUG
//   if (StartX < 0 || StartX > im.GetWidth()-1 ||
//       StartY < 0 || StartY > im.GetHeight()-1 ||
//       EndX < 0 || EndX > im.GetWidth()-1 ||
//       EndY < 0 || EndY >im.GetHeight()-1)
//     BIASERR("wrong coordinates");
// #endif

//   StorageType **ida=im.GetImageDataArray();
//   int DrawX, DrawY, StX=StartX, StY=StartY, EnX=EndX, EnY=EndY;
//   int OffsetX, OffsetY;
//   bool _bxinv, _byinv, _over_angle;
//   // seeding point, rise, ...
//   int X = 0, Y = 0;
//   int dY,dX, dYmX;
//   int E;
//   int tmp;

//   // shift seeding point to origin and transform
//   // line to first quadrant
//   //
//   OffsetX=StX;
//   OffsetY=StY;
//   StX=0; StY=0;
//   EnX=EnX-OffsetX;
//   EnY=EnY-OffsetY;

//   // transform line to first quadrant
//   if (EnX < 0) { EnX = -EnX; _bxinv = true; }
//   else _bxinv = false;

//   if (EnY < 0) { EnY = -EnY; _byinv = true; }
//   else _byinv = false;

//   // rise less equal 1
//   if (EnX < EnY) {
//     tmp = EnX; EnX = EnY; EnY = tmp;
//     _over_angle = true;
//   }
//   else _over_angle = false;

//   // calculate rise
//   dX = EnX-StX;
//   dY = EnY-StY;
//   dYmX = dY-dX;

//   E = dYmX+dY;

//   // compute Bresenham points
//   for (int i=0; i<dX; i++) {
//     if (_over_angle) { DrawX = Y; DrawY = X; } 
//     else { DrawX = X; DrawY = Y; }
//     if (_bxinv) DrawX = -DrawX;
//     if (_byinv) DrawY = -DrawY;
//     DrawY+=OffsetY;
//     DrawX+=OffsetX;
//     // draw point
//     ida[DrawY][DrawX]=Value;
//     // image->PutPixel(&GreyValue,XCoord(X,Y),YCoord(X,Y));
//     X++;

//     // calc next y-coordinate
//     if (E > 0) { Y++; E += 2*(dYmX); }
//     else  E += 2*dY;
//   }
//   if (_over_angle) { DrawX = Y; DrawY = X; }
//   else { DrawX = X; DrawY = Y; }
//   if (_bxinv) DrawX = -DrawX;
//   if (_byinv) DrawY = -DrawY;
//   DrawY+=OffsetY;
//   DrawX+=OffsetX;
//   ida[DrawY][DrawX]=Value;
//   // image->PutPixel(&GreyValue,XCoord(X,Y),YCoord(X,Y));
//   return 0;
// }


////////////////////////////////////////////////////
// interpolated lines
///////////////////////////////////////////////////////

template <class StorageType>
int ImageDraw<StorageType>::InterpolatedLine(Image<StorageType>& im, 
                                                 const int StartX, 
                                                 const int StartY,
                                                 const int EndX, 
                                                 const int EndY, 
                                                 StorageType Value[])
{
  if (im.GetChannelCount()==1){
    return InterpolatedLineGrey(im, StartX, StartY, EndX, EndY, Value[0]);
  } else {
    BIASERR("DrawInterpolatedLineColor not jet implemented");
    return -1;
  }
}

template <class StorageType>
int ImageDraw<StorageType>::InterpolatedLineGrey(Image<StorageType>& im, 
                                                     const int StartX, 
                                                     const int StartY,
                                                     const int EndX, 
                                                     const int EndY, 
                                                     StorageType GreyValue)
{
  StorageType **ida = im.GetImageDataArray();
  EightWaySymmetry<int> sym;
  enum TLineType type;
  int TStartX, TStartY, TEndX, TEndY, CurrX, CurrY;
  double TSlope;

  type = sym.DetermineLineType(StartX, StartY, EndX, EndY);

  sym.Transform(type, StartX, StartY, TStartX, TStartY);

  sym.Transform(type, EndX, EndY, TEndX, TEndY);

  TSlope = (double)(TEndY - TStartY)/(double)(TEndX - TStartX);

  //    cerr << "TSlope: " << TSlope << endl;

  register double g, g1, g2, gm, gtmp;
  g = 0.0;
  g1 = TSlope - 1.0;
  g2 = TSlope;

  if (TSlope > 1e-15){
    while (TStartX < TEndX) {
      
      if (g>1.0){
        g+=g1;
        gtmp = (g<1.0)?(g):(1.0);
        TStartY++;
        gm = (g<1.0)?(1.0 - g):(0.0);
        sym.InverseTransform(type, TStartX, TStartY, CurrX, CurrY);
        //  	cerr << " ++a type: " <<type << "  x " << CurrX << "  y " << CurrY << "   g " << g << "  gtmp " << "  g1: " << g1 << "  g2: " << g2 << endl;
        ida[CurrY][CurrX] = (StorageType)rint((double)GreyValue * gm) +
          (StorageType)rint((double)ida[CurrY][CurrX] * gtmp);
        sym.InverseTransform(type, TStartX, TStartY+1, CurrX, CurrY);
        //  	cerr << " ++b type: " <<type << "  x " << CurrX << "  y " << CurrY << "   gm " << gm << "  gtmp " << gtmp << endl;
        ida[CurrY][CurrX] = (StorageType)rint((double)GreyValue * gtmp) + 
          (StorageType)rint((double)ida[CurrY][CurrX] * gm);
        TStartX++;
      } else {
        gm = 1.0 - g;
        sym.InverseTransform(type, TStartX, TStartY, CurrX, CurrY);
        //  	cerr << " a type: " <<type << " x " << CurrX << "  y " << CurrY << "   g " << g << endl;
        ida[CurrY][CurrX] = (StorageType)rint((double)GreyValue * gm) + 
          (StorageType)rint((double)ida[CurrY][CurrX] * g);
        sym.InverseTransform(type, TStartX, TStartY+1, CurrX, CurrY);
        //  	cerr << " b type: " <<type <<" x " << CurrX << "  y " << CurrY << "   gm " << gm << endl;
        ida[CurrY][CurrX] = (StorageType)rint((double)GreyValue * g) + 
          (StorageType)rint((double)ida[CurrY][CurrX] * gm);
        TStartX++;
        g+=g2;
      }
    }
  } else {
    while (TStartX < TEndX){
      //cerr << " x " << TStartX << "  y " << StartY << "   g " << g << endl;
      sym.InverseTransform(type, TStartX, TStartY, CurrX, CurrY);
      ida[CurrY][CurrX] = (StorageType)rint((double)GreyValue);
      TStartX++;
    }
  }
  return 0;
}

///////////////////////////////////////////////////////////
// circles
//////////////////////////////////////////////////////////

template <class StorageType>
int ImageDraw<StorageType>::CircleCenter(Image<StorageType>& im,
                                         unsigned int CenterX, 
                                         unsigned int CenterY, 
                                         unsigned int Radius, 
                                         StorageType Value[])
{
  int res=0;
  BresenhamCircle bc;
  int center[2], next[2], radius=(int)Radius;
  StorageType **ida=im.GetImageDataArray();
  unsigned int cc=im.GetChannelCount();  
  register unsigned int c;

  center[0]=(int)CenterX;
  center[1]=(int)CenterY;

  bc.Init(center, radius);

  while (bc.GetNext(next)){
    if (next[1]>=0 && next[1]<(int)im.GetHeight() && 
        next[0]>=0 && next[0]<(int)im.GetWidth()) {
      for (c=0; c<cc; c++) {
        ida[next[1]][next[0]*cc+c]=Value[c];
      }
    }
  }

  return res;
}

template <class StorageType>
int ImageDraw<StorageType>::CircleCenterFilled(Image<StorageType>& im,
                                               unsigned int CenterX, 
                                               unsigned int CenterY, 
                                               unsigned int Radius, 
                                               StorageType Value[])
{
  int res=0;
  BresenhamCircleEighth bc;
  int zerocenter[]={0,0};
  int center[2], next[2], radius=(int)Radius;
  unsigned int start[2], end[2];

  center[0]=(int)CenterX;
  center[1]=(int)CenterY;

  bc.Init(zerocenter, radius);

  while (bc.GetNext(next)){
    start[0]=center[0]+next[0]; start[1]=center[1]+next[1];
    end[0]=center[0]-next[0]; end[1]=center[1]+next[1];
    Line(im, start, end, Value);
    start[0]=center[0]+next[1]; start[1]=center[1]+next[0];
    end[0]=center[0]-next[1]; end[1]=center[1]+next[0];
    Line(im, start, end, Value);
    start[0]=center[0]+next[0]; start[1]=center[1]-next[1];
    end[0]=center[0]-next[0]; end[1]=center[1]-next[1];
    Line(im, start, end, Value);
    start[0]=center[0]+next[1]; start[1]=center[1]-next[0];
    end[0]=center[0]-next[1]; end[1]=center[1]-next[0];
    Line(im, start, end, Value);
  }

  return res;
}


template <class StorageType>
int ImageDraw<StorageType>::Arrow(Image<StorageType>& im,
                                  const unsigned start[2],
                                  const unsigned end[2],
                                  const unsigned length, 
                                  const unsigned width, 
                                  StorageType value[])
{
  int dx=(int)end[0]-(int)start[0], dy=(int)end[1]-(int)start[1];

  if (Line(im, start, end, value)!=0){
    BIASERR("error drawing line");
    return -1;
  }

#ifdef BIAS_DEBUG
  if (sqrt(double(dx*dx+dy*dy))<length || sqrt(double(dx*dx+dy*dy))<width){
    BIASERR("funny arrow, head bigger than tail");
    return -2;
  }
#endif  
  int next[2];
  Bresenham bres(end, start), bres2;
  for (unsigned i=0; i<length; i++)
    bres.GetNext(next);
  
  int left[2], right[2], lend[2], rend[2];
  rend[0]=(int)next[0]-dy;
  rend[1]=(int)next[1]+dx;
  lend[0]=(int)next[0]+dy;
  lend[1]=(int)next[1]-dx;
  //cerr << "next: ("<<next[0]<<", "<<next[1]<<")  lend: ("<<lend[0]<<", "
  //     <<lend[1]<<")  rend: ("<<rend[0]<<", "<<rend[1]<<")\n";
  bres.Init(next, rend);
  bres2.Init(next, lend);
  for (unsigned i=0; i<width; i++){
    bres.GetNext(right);
    bres2.GetNext(left);
  }

  if (Line(im, end, (unsigned *)right, value)!=0){
    BIASERR("error drawing line");
    return -1;
  };
  if (Line(im, end, (unsigned *)left, value)!=0){
    BIASERR("error drawing line");
    return -1;
  };
  return 0;
}
