/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003, 2004    (see file CONTACTS for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include "Matrix3x4.hh"

using namespace BIAS;
using namespace std;


template <class T>
Matrix3x4<T>::~Matrix3x4() {}

template<class T>
Matrix3x4<T>& Matrix3x4<T>::operator= (const Matrix<T> &mat) {
    if ((mat.num_rows() == 3) && (mat.num_cols() == 4)) {
        Matrix<T>::operator= (mat);	// call the operator of the base class
        return *this;
    } else {
        BIASERR("cannot assign with operator= , num_rows=" << mat.num_rows() << " num_cols= " << mat.num_rows()
               << " are'nt appropriate for a 3x4 Matrix3x4<T>.");
#if DEBUG
        exit(-1);
#endif
    };
    return *this;
}

template
<class T>
Matrix3x4<T> & Matrix3x4<T>::newsize(int rows, int cols) {
    if ((rows != 3) || (cols != 4)) {
        // do nothing
        // print error message if the new size should not be a 3x4 matrix
        BIASERR("The size of a Matrix3x4<T> has to be 3 rows, 4 cols. newsize can't create a " <<rows<< " x " <<cols<< " Matrix3x4 !");
#if DEBUG
        exit(-1);
#endif
        return *this;

    } else {
        BIASASSERT( rows==3 ); // 3x4 matrix
        BIASASSERT( cols==4 );
        // call base class function (usually unnecessary except for first construction)
        Matrix<T>::newsize(3, 4);
        return *this;
    }
}


