/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003, 2004    (see file CONTACTS for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "CornerDetectorBase.hh"

using namespace BIAS;
using namespace std;

//////////////////////////////////////////////////////////////////////////
//                 instantiation
//////////////////////////////////////////////////////////////////////////

template class CornerDetectorBase<unsigned char>; 
template class CornerDetectorBase<float>; 

// fill in instances as required
#ifdef BUILD_IMAGE_INT
#endif
#ifdef BUILD_IMAGE_CHAR
#endif
#ifdef BUILD_IMAGE_SHORT
#endif
#ifdef BUILD_IMAGE_USHORT
template class CornerDetectorBase<unsigned short>; 
#endif
#ifdef BUILD_IMAGE_UINT
#endif
#ifdef BUILD_IMAGE_DOUBLE
#endif

//////////////////////////////////////////////////////////////////////////
//                 implementation
//////////////////////////////////////////////////////////////////////////


template <class StorageType>
CornerDetectorBase<StorageType>::
CornerDetectorBase() 
{
  ParamsAllocated_ = true;
  _MaxNum = new int; 
  *_MaxNum = 0;
}

template <class StorageType>
CornerDetectorBase<StorageType>::
CornerDetectorBase(Param& para, const std::string& postfix) 
{
  ParamsAllocated_ = false;
  int grp=para.GetFreeGroupID();
  _AddParameter(para, grp, "-CornerDetectorBase"+postfix);
  if (para.IsUsedGroupID(grp)) para.SetGroupName(grp,"CornerDetectorBase");
}

template <class StorageType>
CornerDetectorBase<StorageType>::
CornerDetectorBase(Param& para, const int grp, const std::string& postfix)
{
  ParamsAllocated_ = false;
  _AddParameter(para, grp, "-CornerDetectorBase"+postfix);
}

template <class StorageType>
CornerDetectorBase<StorageType>::
~CornerDetectorBase()
{
  if (ParamsAllocated_) {
    delete _MaxNum;
  }
}

template <class StorageType>
int CornerDetectorBase<StorageType>::
Detect(const Image<StorageType>& image, std::vector<HomgPoint2D>& p, 
       std::vector<QUAL>& quality)
{
  BIASERR("call thsi function from derived class");
  return -10;
}

template <class StorageType>
int *CornerDetectorBase<StorageType>::
AddCornerDetectorType(Param& para, const string& name, const int grp)
{
  vector<string> tt;
  vector<int> tti;
  tt.push_back("CDT_Susan"); 
  tti.push_back(CornerDetectorBase<StorageType>::CDT_Susan);
  tt.push_back("CDT_Harris"); 
  tti.push_back(CornerDetectorBase<StorageType>::CDT_Harris);
  tt.push_back("CDT_Foerstner"); 
  tti.push_back(CornerDetectorBase<StorageType>::CDT_Foerstner);
  tt.push_back("CDT_KLT"); 
  tti.push_back(CornerDetectorBase<StorageType>::CDT_KLT);
  tt.push_back("CDT_Laplace"); 
  tti.push_back(CornerDetectorBase<StorageType>::CDT_Laplace);
  //tt.push_back(""); tti.push_back(CornerDetectorBase<StorageType>::);
  int *res;
  if (para.CheckParam(name)){ 
    res=para.GetParamEnum(name);
  } else { 
    res=para.AddParamEnum(name, 
                          "type of corner detector to use", tt, CDT_KLT,
                          &tti, 0, grp); 
  }
  return res;
}

template <class StorageType>
void CornerDetectorBase<StorageType>::
_AddParameter(Param& para, const int grp, const std::string& postfix) 
{
  if (para.CheckParam("MaxNumCorners"+postfix)) { 
    _MaxNum = para.GetParamInt("MaxNumCorners"+postfix);
  } else { 
    _MaxNum = para.AddParamInt("MaxNumCorners"+postfix, 
                               "Maximum number of corners to detect", 
                               0, 0, INT_MAX, 0, grp);
  }
}
