/*
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003, 2004    (see file CONTACTS for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#ifndef __GradientGaussAsymmetric_hh__
#define __GradientGaussAsymmetric_hh__

#include "GradientGauss.hh"

#include <Base/Math/Vector3.hh>

  // the debugfasgs are defined in Convolution.hh
  // #define D_GRADGAUSS_KERNEL 0x00000001

namespace BIAS {

  /** @class GradientGaussAsymmetric
      @ingroup g_filter
      @brief gradient calculation with separated gauss masks

      @author woelk 09/2004 */
  template <class InputStorageType, class OutputStorageType> 
  class GradientGaussAsymmetric 
    : public GradientGauss<InputStorageType, OutputStorageType>
  {
  public:
    GradientGaussAsymmetric();

    GradientGaussAsymmetric(Param & para, const std::string& postfix="");

    GradientGaussAsymmetric(Param & para, const int grp, 
                            const std::string& postfix);

    virtual ~GradientGaussAsymmetric();

    /** returns a 2 channel image containing gx and gy
        @author woelk 12/2004, untested */
    virtual int Filter(const Image<InputStorageType>& src, 
                       Image<OutputStorageType>& grad);

    virtual int Filter(const Image<InputStorageType>& src, 
                       Image<OutputStorageType>& gx,
                       Image<OutputStorageType>& gy);

    virtual int Filter(const Image<InputStorageType>& src, 
                       Image<OutputStorageType>& gx,
                       Image<OutputStorageType>& gy, 
                       Image<OutputStorageType>& absg);
   
  protected:
    /** Fills _HVec with gaussian function and _VVec with derivative of 
        gaussian function with standard deviation Sigma. The size of _HVec 
        and _VVec is calculated by using Ration. The Gaussian function is 
        truncated, when the ration between biggest and smalles entry falls
        below Ratio.
        @author woelk 09/2004 */
    void _CalculateKernels(double Sigma, double Ratio);

  }; // class


} // namsepace

#endif // __GradientGaussAsymmetric_hh__
