/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003, 2004    (see file CONTACTS for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#ifndef __BIAS_EMatrix_hh__
#define __BIAS_EMatrix_hh__
#include "bias_config.h" 


#include <Base/Debug/Error.hh>
#include <Base/Debug/Debug.hh>
#include "./FMatrix.hh"
#include "./RMatrix.hh"
#include "./Triangulation.hh"

namespace BIAS {
  /** @class EMatrix
      @ingroup g_geometry
      @brief class representing an Essential matrix

      A essential matrix describes the orientation and position of 
      two image planes with known calibration. It can be used to estimate 
      the epipolarline and to distinguish between mismatches and 
      probably good matchesbecause there are some type of missmatches which
      are consistent with the epipolar geometry.
      
      @todo create connection to Base/Geometry/EMatrix
  */
  class EMatrix : public FMatrix
  {
  public:

    EMatrix() : FMatrix(){};
    
    EMatrix(const FMatrixBase &F,  KMatrix &K1,  KMatrix &K2)
    { InitFromF(F,K1,K2); }

    /** @brief assuming K1 and K2 as calibrations of the cameras which refer to
        the fmatrix, create the nearest valid essential matrix from F */
    void InitFromF(const FMatrixBase &F, KMatrix &K1, KMatrix &K2);
    
    /** @brief find R and C(direction), such that maximum number of 3d points
        trinagulated from correspondences inlier1 and inlier2 are in front of
        both cameras 
        @return 0=ok, <0 on error, e.g. no triangulation possible */
    int GetRotationTranslation(RMatrix &R, Vector3<double> &C,
                               std::vector<HomgPoint2D> &inlier1,
                               std::vector<HomgPoint2D> &inlier2);

    inline EMatrix& operator=(const Matrix3x3<FMATRIX_TYPE>& f){
      Matrix3x3<double>::operator=(f);
      return *this; 
    };
    
  };


}

#endif

