#include <wx/wx.h>

#include <Gui/ImageCanvasEnhanced.hh>
#include <Base/Image/ImageIO.hh>

using namespace BIAS;
using namespace std;

enum { ID_Quit = 1, ID_About, ID_Load, ID_Rect};

class MyApp: public wxApp
{
  virtual bool OnInit();
};

IMPLEMENT_APP(MyApp);

class MyFrame: public wxFrame, public ICECallbackInterface
{
public:

  MyFrame(const wxString& title, const wxPoint& pos, const wxSize& size);

  void OnQuit(wxCommandEvent& event);
  void OnAbout(wxCommandEvent& event);
  void OnLoad(wxCommandEvent& event);
  void OnRect(wxCommandEvent& event);

  virtual void RectangleSelected(int tl[2], int br[2]);

  ImageCanvasEnhanced *_ic;

  DECLARE_EVENT_TABLE();
};

bool MyApp::OnInit()
{
  MyFrame *frame = new MyFrame( "ImageCanvasEnhanced", wxPoint(50,50), 
                                wxSize(450,340) );
  frame->Show(TRUE);
  SetTopWindow(frame);
  return TRUE;
} 

BEGIN_EVENT_TABLE(MyFrame, wxFrame)
  EVT_MENU(ID_Quit, MyFrame::OnQuit)
  EVT_MENU(ID_About, MyFrame::OnAbout)
  EVT_MENU(ID_Load, MyFrame::OnLoad)
  EVT_MENU(ID_Rect, MyFrame::OnRect)
END_EVENT_TABLE()


MyFrame::MyFrame(const wxString& title, const wxPoint& pos, const wxSize& size)
  : wxFrame((wxFrame *)NULL, -1, title, pos, size)
{
  wxMenu *menuFile = new wxMenu;
  
  menuFile->Append( ID_About, "&About..." );
  menuFile->Append( ID_Load, "&Load..." );
  menuFile->Append( ID_Rect, "&Rect..." );
  menuFile->AppendSeparator();
  menuFile->Append( ID_Quit, "E&xit" );

  wxMenuBar *menuBar = new wxMenuBar;
  menuBar->Append(menuFile, "&File");
  SetMenuBar(menuBar);

  CreateStatusBar(2);
  SetStatusText("ImageName", 0);

  wxBoxSizer *box=new wxBoxSizer(wxVERTICAL);
  _ic=new ImageCanvasEnhanced(this, GetStatusBar(), 1);
  box->Add(_ic, 1, wxALL| wxGROW |wxALIGN_CENTER, 5);
  box->SetSizeHints(this);
  SetSizer(box);

  Image<unsigned char> im;
  ImageIO::Load("ucim.pgm", im);
  SetStatusText("ucim.pgm", 0);
  _ic->Show(im, "ucim.pgm");
  FitInside();
}

void MyFrame::OnQuit(wxCommandEvent& WXUNUSED(event))
{ Close(TRUE); }

void MyFrame::OnAbout(wxCommandEvent& WXUNUSED(event))
{
  wxMessageBox("This is a wxWindows Hello world sample",
             "About Hello World", wxOK | wxICON_INFORMATION, this);
}

void MyFrame::OnLoad(wxCommandEvent& WXUNUSED(event))
{
  wxFileDialog fileDialog(this, "Choose a file", ".", "ucim.pgm");
  if (fileDialog.ShowModal() == wxID_OK){
    string fname=fileDialog.GetPath().c_str();
    Image<unsigned char> im;
    if (ImageIO::Load(fname, im)!=0){
      BIASERR("errro loading "<<fname);
    } else {
      SetStatusText(fileDialog.GetFilename().c_str(), 0);
      _ic->Show(im, fileDialog.GetFilename().c_str());
    }
  } else {
    cerr << "canceled loadin image" << endl;
  }
}

void MyFrame::OnRect(wxCommandEvent& WXUNUSED(event))
{
  _ic->SelectRectangle(*this);
}
 
void MyFrame::RectangleSelected(int tl[2], int br[2])
{
  cerr << "finished select rectangle: ("<<tl[0]<<", "<<tl[1]<<")  <->  ("
       <<br[0]<<", "<<br[1]<<")\n";
}
