#include "ParamGUI.hh"

// STL headers
#include <iostream>
#include <map>

// BIAS headers
#include <Utils/ParamTypes.hh>
#include <Base/Debug/DebugSimple.hh>

// MIP headers

// WX headers
#include <wx/wx.h>
#include <wx/spinctrl.h>

using namespace std;
using namespace BIAS;

// we set a typedef here, as we need const iterators very often...
typedef map<const std::string,BIAS::ParamTypeBase*>::const_iterator CI;

// ========================================================================
// =========                                                      =========
// ====                      P a r a m G U I                           ====
// =========                                                      =========
// ========================================================================
ParamGUI::ParamGUI()
  : Param()
{
}


ParamGUI::~ParamGUI()
{
}


wxPanel* ParamGUI::GetPanel(wxWindow *parent, int GroupID)
{
  ParamGUIPanel *panel = new ParamGUIPanel(parent);
  
  int count = 0;
  for(CI it=ParamMap_.begin(); it!=ParamMap_.end(); it++){
    if ( it->second->Group_ == GroupID ) {
      //   panel->SetToolTip("Hallo");
      count++;
      if ( ParamTypeInt *item =
	   dynamic_cast<ParamTypeInt*>(it->second) )
	panel->AddIntWidgets(item);
      else if ( ParamTypeBool *item = 
		dynamic_cast<ParamTypeBool*>(it->second) )
	panel->AddBoolWidgets(item);
      else if ( ParamTypeDouble *item =
		dynamic_cast<ParamTypeDouble*>(it->second) )
	panel->AddDoubleWidgets(item);
      else if ( ParamTypeString *item =
		dynamic_cast<ParamTypeString*>(it->second) )
	panel->AddStringWidgets(item);
      else if ( ParamTypeVecDbl *item =
		dynamic_cast<ParamTypeVecDbl*>(it->second) )
	panel->AddVecDblWidgets(item);
      else if ( ParamTypeVecInt *item =
		dynamic_cast<ParamTypeVecInt*>(it->second) )
	panel->AddVecIntWidgets(item);
      else if ( ParamTypeEnum *item =
		dynamic_cast<ParamTypeEnum*>(it->second) )
	panel->AddEnumWidgets(item);
      else
	{
	  BIASERR("Invalid parameter type!");
	}
    }
  }
  if ( count>0 ){
    wxSize s = panel->GetBestSize();
    cout <<"Best Size is : "<<s.GetWidth()<<" x "<<s.GetHeight()<<endl;
    panel->SetVirtualSize(s);
    return panel;
  }
  else
    {
      BIASERR("The requested group has no members!");
      delete panel;
      return NULL;
    }
}


wxNotebook* ParamGUI::GetNotebook(wxWindow *parent)
{
//   cout << "This is ParamGUI::GetNotebook()." << endl;
  wxNotebook *notebook = new wxNotebook( parent, -1 );

  std::vector<int> groups = UniqueSortedGroups();
  
  wxPanel *panel;
  wxString str;
  for(unsigned int i=0; i<groups.size(); i++)
    if (groups[i] != GRP_NOSHOW) {
      cout << "Building panel for group " << groups[i] << endl;
      panel = GetPanel(notebook, groups[i]);
      if(GroupNames_.size()<=groups.size()){
        str.Printf("Group %d",groups[i]);
      }
      else
      {
        str = (GetGroupName(groups[i])).c_str();
      }
      if ( ! notebook->AddPage( panel, str ) ){
	BIASERR("Error adding notebook page!");
	delete notebook;
	return NULL;
      }
    }
  notebook->Fit();
  return notebook;
}


wxFrame* ParamGUI::GetFrame(wxWindow *parent)
{
//   cout << "This is ParamGUI::GetFrame()." << endl;
  wxFrame *newframe = new wxFrame(parent, -1, _T("ParamGUI Frame"), 
    wxDefaultPosition, wxDefaultSize, 
    wxFRAME_TOOL_WINDOW);
  wxNotebook *MyNotebook  = GetNotebook(newframe);
  wxNotebookSizer *nbsizer;
  nbsizer = new wxNotebookSizer(MyNotebook);
  newframe->SetSizer(nbsizer);
  newframe->SetSize(wxSize(500,500));
  return newframe;
}


// ========================================================================
// =========                                                      =========
// ====                  P a r a m G U I P a n e l                     ====
// =========                                                      =========
// ========================================================================
ParamGUIPanel::ParamGUIPanel(wxWindow *parent)
  : wxScrolledWindow(parent)
{
  Parent_ = parent;
  WidgetIDs_.clear();
  Grid_ = new wxFlexGridSizer(1,3,2,5);

  wxStaticText *Header1 = new wxStaticText( this, -1, _T("Parameter") );
  wxStaticText *Header2 = new wxStaticText( this, -1, _T("Type") );
  wxStaticText *Header3 = new wxStaticText( this, -1, _T("Value") );
  Grid_->Add(Header1, 0, wxALIGN_CENTER);
  Grid_->Add(Header2, 0, wxALIGN_CENTER);
  Grid_->Add(Header3, 0, wxALIGN_CENTER);
  SetScrollRate(1,1);
  SetSizer(Grid_);
}


ParamGUIPanel::~ParamGUIPanel()
{
}


BEGIN_EVENT_TABLE(ParamGUIPanel, wxPanel)
  EVT_SPINCTRL          (-1, ParamGUIPanel::OnSpinCtrl)
  EVT_CHECKBOX          (-1, ParamGUIPanel::OnCheckBox)
  EVT_TEXT_ENTER        (-1, ParamGUIPanel::OnTextEnter)
  EVT_CHOICE            (-1, ParamGUIPanel::OnChoice)
END_EVENT_TABLE()


void ParamGUIPanel::AddIntWidgets(ParamTypeInt *item)
{
  wxString tooltip(item->Help_.c_str());
  unsigned int ID = wxNewId(); // generate a new unique ID
  wxStaticText *LabelText = new wxStaticText( this, -1, item->Name_.c_str() );
  wxStaticText *TypeText  = new wxStaticText( this, -1, _T("Integer") );
  wxString str;
  str.Printf("%d",item->value_);
  wxSpinCtrl *Spin = new wxSpinCtrl( this, ID, str, wxDefaultPosition,
                                     wxDefaultSize, wxSP_ARROW_KEYS,
                                     item->min_, item->max_ );
  Spin->SetToolTip(tooltip);
  Grid_->SetRows(Grid_->GetRows()+1);
  Grid_->Add(LabelText, 0, wxALIGN_CENTER);
  Grid_->Add(TypeText, 0, wxALIGN_CENTER);
  Grid_->Add(Spin, 0, wxALIGN_CENTER);
  Fit();
  struct ItemAndWidget NewStruct;
  NewStruct.widget = Spin;
  NewStruct.item   = item;
  pair<unsigned int, struct ItemAndWidget> NewPair( ID, NewStruct );
  WidgetIDs_.insert(WidgetIDs_.end(), NewPair );
}


void ParamGUIPanel::AddBoolWidgets(BIAS::ParamTypeBool *item)
{
  wxString tooltip(item->Help_.c_str());
  unsigned int ID = wxNewId(); // generate a new unique ID
  wxStaticText *LabelText = new wxStaticText( this, -1, item->Name_.c_str() );
  wxStaticText *TypeText  = new wxStaticText( this, -1, _T("Boolean") );
  wxCheckBox *CheckBox = new wxCheckBox( this, ID, wxEmptyString );
  CheckBox->SetValue(item->value_);

  CheckBox->SetToolTip(tooltip);

  Grid_->SetRows(Grid_->GetRows()+1);
  Grid_->Add(LabelText, 0, wxALIGN_CENTER);
  Grid_->Add(TypeText, 0, wxALIGN_CENTER);
  Grid_->Add(CheckBox, 0, wxALIGN_CENTER);

  struct ItemAndWidget NewStruct;
  NewStruct.widget = CheckBox;
  NewStruct.item   = item;
  pair<unsigned int, struct ItemAndWidget> NewPair( ID, NewStruct );
  WidgetIDs_.insert(WidgetIDs_.end(), NewPair );
}


void ParamGUIPanel::AddDoubleWidgets(BIAS::ParamTypeDouble *item)
{
  wxString tooltip(item->Help_.c_str());
  unsigned int ID = wxNewId(); // generate a new unique ID
  wxStaticText *LabelText = new wxStaticText( this, -1, item->Name_.c_str() );
  wxStaticText *TypeText  = new wxStaticText( this, -1, _T("Double") );
  wxString str;
  str.Printf("%f",item->value_);
  wxTextCtrl *TextCtrl = new wxTextCtrl( this, ID, str, wxDefaultPosition,
                                         wxDefaultSize, wxTE_PROCESS_ENTER );
  TextCtrl->SetToolTip(tooltip);

  Grid_->SetRows(Grid_->GetRows()+1);
  Grid_->Add(LabelText, 0, wxALIGN_CENTER);
  Grid_->Add(TypeText, 0, wxALIGN_CENTER);
  Grid_->Add(TextCtrl, 0, wxALIGN_CENTER);

  struct ItemAndWidget NewStruct;
  NewStruct.widget = TextCtrl;
  NewStruct.item   = item;
  pair<unsigned int, struct ItemAndWidget> NewPair( ID, NewStruct );
  WidgetIDs_.insert(WidgetIDs_.end(), NewPair );
}


void ParamGUIPanel::AddStringWidgets(BIAS::ParamTypeString *item)
{
  wxString tooltip(item->Help_.c_str());
  unsigned int ID = wxNewId(); // generate a new unique ID
  wxStaticText *LabelText = new wxStaticText( this, -1, item->Name_.c_str() );
  wxStaticText *TypeText  = new wxStaticText( this, -1, _T("String") );
  wxString str = item->value_.c_str();
  wxTextCtrl *TextCtrl = new wxTextCtrl( this, ID, str, wxDefaultPosition,
                                         wxSize(200,-1), wxTE_PROCESS_ENTER );
  TextCtrl->SetToolTip(tooltip);
  Grid_->SetRows(Grid_->GetRows()+1);
  Grid_->Add(LabelText, 0, wxALIGN_CENTER);
  Grid_->Add(TypeText, 0, wxALIGN_CENTER);
  Grid_->Add(TextCtrl, 0, wxALIGN_CENTER);

  struct ItemAndWidget NewStruct;
  NewStruct.widget = TextCtrl;
  NewStruct.item   = item;
  pair<unsigned int, struct ItemAndWidget> NewPair( ID, NewStruct );
  WidgetIDs_.insert(WidgetIDs_.end(), NewPair );
}


void ParamGUIPanel::AddVecDblWidgets(BIAS::ParamTypeVecDbl *item)
{  
  wxString tooltip(item->Help_.c_str());
  unsigned int ID = wxNewId(); // generate a new unique ID
  wxStaticText *LabelText = new wxStaticText( this, -1, item->Name_.c_str() );
  wxStaticText *TypeText  = new wxStaticText( this, -1, _T("VecDbl") );
  wxString str = item->GetValueString().c_str();
  wxTextCtrl *TextCtrl = new wxTextCtrl( this, ID, str, wxDefaultPosition,
                                         wxSize(200,-1), wxTE_PROCESS_ENTER );
  TextCtrl->SetToolTip(tooltip);
  Grid_->SetRows(Grid_->GetRows()+1);
  Grid_->Add(LabelText, 0, wxALIGN_CENTER);
  Grid_->Add(TypeText, 0, wxALIGN_CENTER);
  Grid_->Add(TextCtrl, 0, wxALIGN_CENTER);

  struct ItemAndWidget NewStruct;
  NewStruct.widget = TextCtrl;
  NewStruct.item   = item;
  pair<unsigned int, struct ItemAndWidget> NewPair( ID, NewStruct );
  WidgetIDs_.insert(WidgetIDs_.end(), NewPair );
}


void ParamGUIPanel::AddVecIntWidgets(BIAS::ParamTypeVecInt *item)
{
  wxString tooltip(item->Help_.c_str());
  unsigned int ID = wxNewId(); // generate a new unique ID
  wxStaticText *LabelText = new wxStaticText( this, -1, item->Name_.c_str() );
  wxStaticText *TypeText  = new wxStaticText( this, -1, _T("VecInt") );
  wxString str = item->GetValueString().c_str();
  wxTextCtrl *TextCtrl = new wxTextCtrl( this, ID, str, wxDefaultPosition,
                                         wxSize(200,-1), wxTE_PROCESS_ENTER );
  TextCtrl->SetToolTip(tooltip);
  Grid_->SetRows(Grid_->GetRows()+1);
  Grid_->Add(LabelText, 0, wxALIGN_CENTER);
  Grid_->Add(TypeText, 0, wxALIGN_CENTER);
  Grid_->Add(TextCtrl, 0, wxALIGN_CENTER);

  struct ItemAndWidget NewStruct;
  NewStruct.widget = TextCtrl;
  NewStruct.item   = item;
  pair<unsigned int, struct ItemAndWidget> NewPair( ID, NewStruct );
  WidgetIDs_.insert(WidgetIDs_.end(), NewPair );
}


void ParamGUIPanel::AddEnumWidgets(BIAS::ParamTypeEnum *item)
{
  wxString tooltip(item->Help_.c_str());
  unsigned int ID = wxNewId(); // generate a new unique ID
  wxStaticText *LabelText = new wxStaticText( this, -1, item->Name_.c_str() );
  wxStaticText *TypeText  = new wxStaticText( this, -1, _T("Enum") );
  int n = item->Map_.size();
  int i=0;
  wxString *choices=new wxString[n];
  for(map<string,int>::iterator it=item->Map_.begin();
      it!=item->Map_.end(); it++){
      choices[i] = it->first.c_str();
      i++;
    }
  wxChoice *Choice = new wxChoice(this, ID, wxDefaultPosition, wxSize(200,-1),
                                  n, choices );
  Choice->SetToolTip(tooltip);

  Grid_->SetRows(Grid_->GetRows()+1);
  Grid_->Add(LabelText, 0, wxALIGN_CENTER);
  Grid_->Add(TypeText, 0, wxALIGN_CENTER);
  Grid_->Add(Choice, 0, wxALIGN_CENTER);

  struct ItemAndWidget NewStruct;
  NewStruct.widget = Choice;
  NewStruct.item   = item;
  pair<unsigned int, struct ItemAndWidget> NewPair( ID, NewStruct );
  WidgetIDs_.insert(WidgetIDs_.end(), NewPair );
  delete[] choices;
}


void ParamGUIPanel::OnSpinCtrl(wxSpinEvent &event)
{
//   COUT("Fetched wxSpinEvent!" << endl);
  int ID = event.GetId();
//   COUT("ID = " << ID << endl);
  map<unsigned int,struct ItemAndWidget>::const_iterator it;
  it = WidgetIDs_.find(ID);
  if ( it == WidgetIDs_.end() ){
      BIASERR("This error should never occur or otherwise\n "<<
              "something with the event IDs went wrong");
      return;
    }
  if ( ParamTypeInt *item = 
       dynamic_cast<ParamTypeInt*>(it->second.item) ) {
    item->value_ = ((wxSpinCtrl*)(it->second.widget))->GetValue();
    if (item->CallbackObject_ != NULL)
      item->CallbackObject_->ParameterChanged(item->Name_, &item->value_);
  }
  else
    {
      BIASERR("Error during dynamic cast!");
    }
}


void ParamGUIPanel::OnCheckBox(wxCommandEvent &event)
{
//   COUT("Fetched CheckBox event!" << endl);
  int ID = event.GetId();
//   COUT("ID = " << ID << endl);
  map<unsigned int,struct ItemAndWidget>::const_iterator it;
  it = WidgetIDs_.find(ID);
  if ( it == WidgetIDs_.end() ){
      BIASERR("This error should never occur or otherwise\n "<<
              "something with the event IDs went wrong");
      return;
    }
  if ( ParamTypeBool *item = 
       dynamic_cast<ParamTypeBool*>(it->second.item) ){
    item->value_ = ((wxCheckBox*)(it->second.widget))->GetValue();
    if (item->CallbackObject_ != NULL)
      item->CallbackObject_->ParameterChanged(item->Name_, &item->value_);
  }
  else{
      BIASERR("Error during dynamic cast!");
    }
}


void ParamGUIPanel::OnTextEnter(wxCommandEvent &event)
{
    COUT("Fetched TextEnter event!" << endl);
  int ID = event.GetId();
  //   COUT("ID = " << ID << endl);
  map<unsigned int,struct ItemAndWidget>::const_iterator it;
  it = WidgetIDs_.find(ID);
  if ( it == WidgetIDs_.end() ){
    BIASERR("This error should never occur or otherwise\n "<<
	    "something with the event IDs went wrong");
    return;
  }
  if ( ParamTypeDouble *item = 
       dynamic_cast<ParamTypeDouble*>(it->second.item) ){
    double newval;
    if ( !((wxTextCtrl*)it->second.widget)->GetValue().ToDouble(&newval) ){
      wxMessageBox( "Please check your input!",
		    "Not a double!",
		    wxOK | wxICON_ERROR );
    }
    else{
      // atof(
      // item->SetFromString() <0) 
      // setwidgetvaslue(item->GetValueString();
      if ( newval > item->max_ ){
	wxString error;
	error.Printf("Your value outruns the upper border!\nValue clipped to %f!",
		     item->max_);
	wxMessageBox( error,"Limits!",wxOK | wxICON_ERROR );
	item->value_ = item->max_;
      }
      else if ( newval < item->min_ )
	{
	  wxString error;
	  error.Printf("Your value underruns the lower border!\nValue "
		       "clipped to %f!",item->min_);
	  wxMessageBox( error,"Limits!",wxOK | wxICON_ERROR );
	  item->value_ = item->min_;
	}
      else
	item->value_ = newval;
    if (item->CallbackObject_ != NULL)
      item->CallbackObject_->ParameterChanged(item->Name_, &item->value_);
    }
  }
  else if ( ParamTypeString *item =
	    dynamic_cast<ParamTypeString*>(it->second.item) ) {
    item->value_ = ((wxTextCtrl*)(it->second.widget))->GetValue();
    if (item->CallbackObject_ != NULL)
      item->CallbackObject_->ParameterChanged(item->Name_, &item->value_);
  }
  else if ( ParamTypeVecDbl *item = 
	    dynamic_cast<ParamTypeVecDbl*>(it->second.item) ){
    /// \bug  here a bit SetFromStrng()!!
    BIASERR("unfinished "<<item);
    if (item->CallbackObject_ != NULL)
      item->CallbackObject_->ParameterChanged(item->Name_, &item->value_);
  }
  else if ( ParamTypeVecInt *item = 
	    dynamic_cast<ParamTypeVecInt*>(it->second.item) ){
    /// \bug here a bit SetFromStrng()!!
    BIASERR("unfinished "<<item);
    if (item->CallbackObject_ != NULL)
      item->CallbackObject_->ParameterChanged(item->Name_, &item->value_);
  }
  else{
    BIASERR("Error during dynamic cast!");
  }

}


void ParamGUIPanel::OnChoice(wxCommandEvent &event)
{
//   COUT("Fetched Choice event!" << endl);
  int ID = event.GetId();
//   COUT("ID = " << ID << endl);
  map<unsigned int,struct ItemAndWidget>::const_iterator it;
  it = WidgetIDs_.find(ID);
  if ( it == WidgetIDs_.end() )
    {
      BIASERR("This error should never occur or otherwise\n "<<
              "something with the event IDs went wrong");
      return;
    }
  if ( ParamTypeEnum *item = 
       dynamic_cast<ParamTypeEnum*>(it->second.item) )
    {
      wxString str = ((wxChoice*)(it->second.widget))->GetStringSelection();
      string s = str.c_str();
//       cout << str << endl;
      item->SetFromString(s);
    }
  else
    {
      BIASERR("Error during dynamic cast!");
    }
}

