/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003, 2004    (see file CONTACTS for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifdef WIN32
#  define _USE_MATH_DEFINES
#endif //WIN32

#include <math.h>
#include <Geometry/RMatrix.hh>
#include <Base/Geometry/Quaternion.hh>
#include <Base/Image/ImageConvert.hh>
#include <Base/Image/ImageDraw.hh>
#include <Base/Image/ImageIO.hh>

#include "SphericalUndistortion.hh"

#include <iostream>
#include <limits> // JW


using namespace BIAS;
using namespace MIP;

#define INST(type)\
\
template int SphericalUndistortion::ProjectImage(const Image<type>& src, Image<type>& dst, const double& SizeX, const double& SizeY, const double& FovX, const double& FovY, const double& CenterX, const double& CenterY, const int projectionType, const double& zoomFactor, KMatrix& KMatDst);\
template int SphericalUndistortion::DrawCircles(Image<type>& src, const double& DegreeStep);\
template int SphericalUndistortion::CorrectVignette(Image<type>& src);\
template int SphericalUndistortion::FitCircle(const Image<type>& src, double &CenterX, double &CenterY, double &Radius);\
template int SphericalUndistortion::CalcCircularGradient_(const Image<type>& src, double curCentX, double curCentY, double curRad, double &CentGravX, double &CentGravY, double &curGradSum) const;

// template int SphericalUndistortion::CalcCentGrav_(const Image<type> &src, 
  // double &centerX, double &centerY);



INST(unsigned char);
INST(float);
#ifdef BUILD_IMAGE_INT
INST(int);
#endif
#ifdef BUILD_IMAGE_CHAR
INST(char);
#endif
#ifdef BUILD_IMAGE_SHORT
INST(short);
#endif
#ifdef BUILD_IMAGE_USHORT
INST(unsigned short);
#endif
#ifdef BUILD_IMAGE_UINT
INST(unsigned int);
#endif
#ifdef BUILD_IMAGE_DOUBLE
INST(double);
#endif


//#define LOCDEB

SphericalUndistortion::SphericalUndistortion()
{
  isInitialized_=false;
}
    
SphericalUndistortion::SphericalUndistortion(const SphericalUndistortion& sp) {
  
  AngleCorrX_ = sp.AngleCorrX_;
  AngleCorrY_ = sp.AngleCorrY_;
  
  IlluCorrX_ = sp.IlluCorrX_;
  IlluCorrY_ = sp.IlluCorrY_;
  
  interpolatorAngle_ = sp.interpolatorAngle_;
  interpolatorAngleBack_ = sp.interpolatorAngleBack_;
  interpolatorIllu_ = sp.interpolatorIllu_;
  
  MaxCamAngle_ = sp.MaxCamAngle_;
  CenterX_ = sp.CenterX_;
  CenterY_ = sp.CenterY_;
  radius_ = sp.radius_;
  isInitialized_ = sp.isInitialized_;
}

SphericalUndistortion::SphericalUndistortion(const double& centerX, 
                                             const double& centerY, 
                                             const double& radius,
                                             const vector<double> &AngleCorrX, 
                                             const vector<double> &AngleCorrY,
                                             const vector<double> &IlluCorrX, 
                                             const vector<double> &IlluCorrY) 
{
  Init(centerX, centerY, radius, AngleCorrX, AngleCorrY, IlluCorrX, IlluCorrY);
}

SphericalUndistortion::SphericalUndistortion(const CameraParam& cp) {
  Init(cp);
}

SphericalUndistortion::~SphericalUndistortion() {}

/*
int SphericalUndistortion::InitCalibration(int Calibration) {

  double x,y;

  AngleCorrX_.clear();
  AngleCorrY_.clear();
  IlluCorrX_.clear();
  IlluCorrY_.clear();

  isInitialized_=false;

  switch (Calibration) {
  case (IDENTITY):
    
    x=0; y=1;
    AngleCorrX_.push_back(x); AngleCorrY_.push_back(y); 
    x=M_2_PI; y=1;
    AngleCorrX_.push_back(x); AngleCorrY_.push_back(y); 
   
    x=0; y=1;
    IlluCorrX_.push_back(x); IlluCorrY_.push_back(y); 
    x=M_2_PI; y=1;
    IlluCorrX_.push_back(x); IlluCorrY_.push_back(y); 

    MaxCamAngle_ = M_2_PI;
    isInitialized_=true;
    
    break;
  case (OMNITECH_ORIFL_190_3):

    // Angular distortion correction x=theta, y=theta*distortionFactor
    x=0.000; y=1.000*x;
    AngleCorrX_.push_back(x); AngleCorrY_.push_back(y); 
    x=0.138; y=1.000*x;
    AngleCorrX_.push_back(x); AngleCorrY_.push_back(y); 
    x=0.424; y=0.993*x;
    AngleCorrX_.push_back(x); AngleCorrY_.push_back(y); 
    x=0.757; y=0.974*x;
    AngleCorrX_.push_back(x); AngleCorrY_.push_back(y); 
    x=1.045; y=0.941*x;
    AngleCorrX_.push_back(x); AngleCorrY_.push_back(y); 
    x=1.342; y=0.899*x;
    AngleCorrX_.push_back(x); AngleCorrY_.push_back(y); 
    x=1.436; y=0.881*x;
    AngleCorrX_.push_back(x); AngleCorrY_.push_back(y); 
    x=1.652; y=0.828*x;
    AngleCorrX_.push_back(x); AngleCorrY_.push_back(y);
 
    // Vignette correction x=angle, y=darkening factor
    x=0.005; y=1.000;
    IlluCorrX_.push_back(x); IlluCorrY_.push_back(y); 
    x=0.092; y=1.000;
    IlluCorrX_.push_back(x); IlluCorrY_.push_back(y); 
    x=0.179; y=1.000;
    IlluCorrX_.push_back(x); IlluCorrY_.push_back(y); 
    x=0.266; y=1.003;
    IlluCorrX_.push_back(x); IlluCorrY_.push_back(y); 
    x=0.354; y=1.003;
    IlluCorrX_.push_back(x); IlluCorrY_.push_back(y); 
    x=0.439; y=1.003;
    IlluCorrX_.push_back(x); IlluCorrY_.push_back(y); 
    x=0.527; y=1.003;
    IlluCorrX_.push_back(x); IlluCorrY_.push_back(y); 
    x=0.615; y=1.005;
    IlluCorrX_.push_back(x); IlluCorrY_.push_back(y); 
    x=0.701; y=1.005;
    IlluCorrX_.push_back(x); IlluCorrY_.push_back(y); 
    x=0.788; y=1.005;
    IlluCorrX_.push_back(x); IlluCorrY_.push_back(y); 
    x=0.875; y=1.005;
    IlluCorrX_.push_back(x); IlluCorrY_.push_back(y); 
    x=0.962; y=1.003;
    IlluCorrX_.push_back(x); IlluCorrY_.push_back(y); 
    x=1.049; y=0.998;
    IlluCorrX_.push_back(x); IlluCorrY_.push_back(y); 
    x=1.135; y=0.993;
    IlluCorrX_.push_back(x); IlluCorrY_.push_back(y); 
    x=1.223; y=0.983;
    IlluCorrX_.push_back(x); IlluCorrY_.push_back(y); 
    x=1.310; y=0.966;
    IlluCorrX_.push_back(x); IlluCorrY_.push_back(y); 
    x=1.398; y=0.941;
    IlluCorrX_.push_back(x); IlluCorrY_.push_back(y); 
    x=1.485; y=0.897;
    IlluCorrX_.push_back(x); IlluCorrY_.push_back(y); 
    x=1.572; y=0.828;
    IlluCorrX_.push_back(x); IlluCorrY_.push_back(y); 
    x=1.660; y=0.716;
    IlluCorrX_.push_back(x); IlluCorrY_.push_back(y); 

    MaxCamAngle_ = 1.6581; // 95 deg
    isInitialized_=true;

    break;
  default:
#ifdef BIAS_DEBUG
    BIASERR("Unknown Calibration initialization failed!");
#endif
    x=0; y=1;
    AngleCorrX_.push_back(x); AngleCorrY_.push_back(y); 
    x=M_2_PI; y=1;
    AngleCorrX_.push_back(x); AngleCorrY_.push_back(y); 
    
    x=0; y=1;
    IlluCorrX_.push_back(x); IlluCorrY_.push_back(y); 
    x=M_2_PI; y=1;
    IlluCorrX_.push_back(x); IlluCorrY_.push_back(y); 

    MaxCamAngle_ = M_2_PI;
  }

  interpolatorAngle_.SetKnotPoints(AngleCorrX_);  
  interpolatorAngle_.SetControlPoints(AngleCorrY_); 
  interpolatorAngle_.InitSpline();  

  interpolatorAngleBack_.SetKnotPoints(AngleCorrY_);  
  interpolatorAngleBack_.SetControlPoints(AngleCorrX_);  
  interpolatorAngleBack_.InitSpline();  

  interpolatorIllu_.SetKnotPoints(IlluCorrX_);  
  interpolatorIllu_.SetControlPoints(IlluCorrY_);
  interpolatorIllu_.InitSpline();  
  
  return 0;
}
*/
/*
template <class StorageType>
int SphericalUndistortion::Init(int Calibration, 
                            std::vector<Image<StorageType> > &vecSrc) 
{
  Image<StorageType> avgImg; 
  Average avg;

  if (avg.Calc(vecSrc, avgImg)<0) {
    BIASERR("Error averaging images for sphere initialisation!");
    return -1;
  } else {
#ifdef BIAS_DEBUG
    if ((_liDebugLevel & DEBUG_SPHEREPROJ_AVG)!=0) {
      stringstream ssFilename;
      ssFilename << "AverageImage_" << avgImg.GetUID();
      ImageIO::ExportImage(ssFilename.str(),avgImg, ImageIO::FF_pgm);
    }
#endif
    if (Init(Calibration, avgImg)>=0) {
      return 0;
    }
  }
  return -1;
}
*/
/*
template <class StorageType>
int SphericalUndistortion::Init(int Calibration, const Image<StorageType>& src) 
{
    
  double CenterX, CenterY, Radius;
  
  if (FitCircle(src, CenterX, CenterY, Radius)<0) {
#ifdef BIAS_DEBUG
    BIASERR("Sphere Initialisation failed!");    
#endif
    isInitialized_ = false;
    return -1;
  }

  CenterX_ = CenterX;
  CenterY_ = CenterY;
  radius_  = Radius;

#ifdef BIAS_DEBUG

//cout << "CenterX:" << CenterX_ << "  CenterY:" << CenterY_;
//  cout << "  Radius:" << radius_ << "\n" << "\n" << endl;
    
  StorageType* Color;
  Color = new StorageType[src.GetChannelCount()];
  Color[0]=255;
  for (unsigned int j=1; j<src.GetChannelCount(); j++) {
    Color[j]=0;
  }
  if ((_liDebugLevel & DEBUG_SPHEREPROJ_CIRCLE)!=0) {
    Image<StorageType> img;
    img = src;
    ImageDraw<StorageType>::CircleCenter(img,(unsigned int)rint(CenterX_), 
					 (unsigned int)rint(CenterY_), 
					 (unsigned int)rint(radius_), Color);
    delete [] Color;
    
    stringstream ssFilename;
    ssFilename << "CircleFound";
    ImageIO::Write("borderFound.mip", img, ImageIO::FF_mip);
    //cout << " created: "<<"borderFound.mip"<<endl;
    
  }

#endif

  InitCalibration(Calibration);

  isInitialized_=true;
  return 0;

  // Fisheye first mount approx.
  //CenterX_ = 364;
  //CenterY_= 245;
  //radius_ = 226;
}
*/

int SphericalUndistortion::Init(const CameraParam& cp) {
  if (cp.IsSpherical()) {
    return Init(cp.GetPrincipalX(), cp.GetPrincipalY(), cp.GetRadius(), 
                cp.GetAngleCorrX(), cp.GetAngleCorrY(),
                cp.GetIlluCorrX(), cp.GetIlluCorrY());
  } else {
    isInitialized_=false;
    return -1;
  }
}


int SphericalUndistortion::Init(const double& centerX, const double& centerY, 
                                const double& radius,
                                const std::vector<double> &AngleCorrX, 
                                const std::vector<double> &AngleCorrY,
                                const std::vector<double> &IlluCorrX, 
                                const std::vector<double> &IlluCorrY) {
  
  AngleCorrX_.clear();
  AngleCorrY_.clear();
  IlluCorrX_.clear();
  IlluCorrY_.clear();

  CenterX_ = centerX;
  CenterY_ = centerY;
  radius_ = radius;
  AngleCorrX_ = AngleCorrX;
  AngleCorrY_ = AngleCorrY;
  IlluCorrX_ = IlluCorrX;
  IlluCorrY_ = IlluCorrY;

  if (AngleCorrX_.size()<2 || AngleCorrY_.size()<2 || 
      AngleCorrX_.size()!=AngleCorrY_.size()) {
    isInitialized_=false;
    return -1;
  }
  
  //double a = 0.65;
  //for (unsigned int i=0; i<AngleCorrX_.size(); i++) {
    //cout << "before " << AngleCorrY_[i] << endl;
    //AngleCorrY_[i]=a*AngleCorrY_[i]+(1-a)*AngleCorrX_[i];
    //cout << "after " << AngleCorrY_[i] << endl;
  //}

  double dum;
  interpolatorAngle_.SetKnotPoints(AngleCorrX_);  
  interpolatorAngle_.SetControlPoints(AngleCorrY_); 
  interpolatorAngle_.InitSpline(); 
  interpolatorAngle_.Spline(dum, AngleCorrX[0],4);  
  MaxCamAngle_ = AngleCorrX_[AngleCorrX_.size()-1];

  interpolatorAngleBack_.SetKnotPoints(AngleCorrY_);  
  interpolatorAngleBack_.SetControlPoints(AngleCorrX_);  
  interpolatorAngleBack_.InitSpline(); 
  interpolatorAngleBack_.Spline(dum, AngleCorrY[0],4);  

  if (IlluCorrX_.size()>1 && IlluCorrY_.size()>1 && 
      IlluCorrX_.size()==IlluCorrY_.size()) {
    interpolatorIllu_.SetKnotPoints(IlluCorrX_);  
    interpolatorIllu_.SetControlPoints(IlluCorrY_);
    interpolatorIllu_.InitSpline(); 
    interpolatorIllu_.Spline(dum, IlluCorrX[0],4);  
  } else {
    IlluCorrX_.clear();
    IlluCorrY_.clear();
  }

  isInitialized_=true;
  return 0;
}



int SphericalUndistortion::GetSphereCenter(double& centerX, double& centerY) const
{
  if (isInitialized_) {
    centerX = CenterX_;
    centerY = CenterY_;
    return 0;
  } else {
    return -1;
  }
}

int SphericalUndistortion::GetSphereCenter(double& centerX, double& centerY, double& radius) const
{
  if (isInitialized_) {
    centerX = CenterX_;
    centerY = CenterY_;
    radius = radius_;
    return 0;
  } else {
    return -1;
  }
}

double SphericalUndistortion::GetSphereRadius() const
{
  if (isInitialized_) {
    return radius_;
  } else {
    return -1;
  }
}


double SphericalUndistortion::GetMaxCamAngle() const
{
  if (isInitialized_) {
    return MaxCamAngle_;
  } else {
    return -1;
  }
}

template <class StorageType>
int SphericalUndistortion::ProjectImage(const Image<StorageType>& src, 
                                        Image<StorageType>& dst,
                                        const double& SizeX, 
                                        const double& SizeY, 
                                        const double& FovX, 
                                        const double& FovY,
                                        const double& CenterX, 
                                        const double& CenterY, 
                                        const int projectionType, 
                                        const double& zoomFactor,
                                        KMatrix& KMatDst)
{
  if (!isInitialized_) {
#ifdef BIAS_DEBUG
    BIASERR("Sphere Projections was not initialized!");
#endif
    return -1;
  }

  double viewCenterX = CenterX;
  double viewCenterY = CenterY;
  if (viewCenterX==0) viewCenterX = CenterX_;
  if (viewCenterY==0) viewCenterY = CenterY_;
  
  double imageSizeX = SizeX;
  double imageSizeY = SizeY;
  //cout << "ROI: " << startx << "," << starty 
  //     << "," << endx << "," << endy << endl;
  //cout << "viewCenter: " << viewCenterX << "," << viewCenterY << endl;

  double fovXrad, fovYrad;//, dum;
  if (FovX!=0) {
    fovXrad = FovX/360.0*M_PI;
  } else {
    fovXrad = 0;
  }
  if (FovY!=0) {
    fovYrad = FovY/360.0*M_PI;
  } else {
    fovYrad = 0;
  }

  //DrawCircles(src, 20.0);
  
  // either result image size or FoV is fixed
  KMatrix K;
  K.SetIdentity();
  if (fovXrad!=0 && imageSizeX!=0) {
    K[0][0] = (imageSizeX*zoomFactor)/(tan(fovXrad)*2.0);
  }
  if (fovYrad!=0 && imageSizeY!=0) {
    K[1][1] = (imageSizeY*zoomFactor)/(tan(fovYrad)*2.0);
  } 
  if (K[0][0]==1) {
    K[0][0]=K[1][1];
  }
  if (K[1][1]==1) {
    K[1][1]=K[0][0];
  }
  K[0][2] = (imageSizeX*zoomFactor)/2.0;
  K[1][2] = (imageSizeY*zoomFactor)/2.0;
  //cout << "K:" << K << endl;

  /*if (imageSizeX==0) { 
    //same scale in both directions
    //x-direction is responsible for scale
    HomgPoint2D sp1,sp2, dp1, dp2;
    sp1[0] = sp2[0] = (sizeX*zoomFactor)/2.0;
    sp1[1] = CenterY_-sizeY/2.0;
    sp2[1] = CenterY_+sizeY/2.0;
    sp1[2] = sp2[2] = 1;
    ProjectCoordBack(projectionType, K, sp1, dp1);
    ProjectCoordBack(projectionType, K, sp2, dp2);
    K[1][1] = K[0][0];
    ProjectCoord(projectionType, K, dp1, sp1);
    ProjectCoord(projectionType, K, dp2, sp2);
        imageSizeX = rint(sp2[1]-sp1[1]); 
      }
  if (imageSizeY==0) { 
    imageSizeY = 
      }
  */
  //dst.Release();
  if (dst.IsEmpty()) {
    dst.Init((unsigned int)(imageSizeX*zoomFactor), 
             (unsigned int)(imageSizeY*zoomFactor), 
             src.GetChannelCount());
  }
  //cout << (endx-startx) << "," <<  (endy-starty) << endl;
  StorageType** dataDst=dst.GetImageDataArray();
  
  //cout << "Roi for correct scale: " << startx << "," << starty 
  //     << "," << endx << "," << endy << endl;
  //cout << "K:" << K << endl;

  // go through pixels in dest image (backward mapping)
  for (int y=0; y<imageSizeY*zoomFactor; y++) {
    for (int x=0; x<imageSizeX*zoomFactor; x++) {

      //TransfRectImage2Sphere(x,y,projectionType, zoomFactor, theta, phi);

      double xSrc, ySrc;
      int ret=0; 
      HomgPoint2D Source, Dest;
      Source[0]=(double)x;
      Source[1]=(double)y;
      Source[2]=1;
      ProjectCoordBack(projectionType, K, Source, Dest);
      //ProjectCoord(projectionType, K, Dest, Test);
      //cout << Source << Dest << Test << endl;
      if ((viewCenterX!=0 && viewCenterX!=CenterX_) || 
          (viewCenterY!=0 && viewCenterY!=CenterY_)) {
        double t, p;
        //cout << "viewCenter: " << viewCenterX << "," << viewCenterY << endl;
        ret = TransfCoordImage2Sphere (Dest[0], Dest[1], t, p);
        ret = TransfCoordRotateSphere2Image(t, p, 
                                            viewCenterX, viewCenterY,
                                            xSrc, ySrc);
      } else {
        xSrc=Dest[0];
        ySrc=Dest[1];
      }
      
      if (ret==-1) {
        xSrc=ySrc=0;
      }

      if (xSrc<0 || xSrc>src.GetWidth()-1) xSrc=0;
      if (ySrc<0 || ySrc>src.GetHeight()-1) ySrc=0;
            
      for (unsigned int j=0; j<src.GetChannelCount(); j++) {
        dataDst[y][x * dst.GetChannelCount()+j] = 
          (StorageType) src.BilinearInterpolation(xSrc, ySrc,j);
      }
    }
  }
  
  KMatDst=K;

  return 0;
}

int SphericalUndistortion::ProjectCoord(const int projectionType, 
                                    const KMatrix& kMatrix,
                                    const HomgPoint2D& Source,
                                    HomgPoint2D &Dest)
{

  if (!isInitialized_) {
#ifdef BIAS_DEBUG
    BIASERR("Sphere Projections was not initialized!");
#endif
    return -1;
  }
         
  
  double theta,phi;
  TransfCoordImage2Sphere (Source, theta, phi);
  //cout << "ProjectCoord Src:" << Source << "," << theta 
  //<< "," << phi << endl;
 
  double rDst;
  switch (projectionType) {
  case (PERSPECTIVE):
    rDst = tan(theta);
    break; 
  case (STEREOGRAPHIC):
    rDst = tan(theta/2);
    break; 
  case (EQUIDISTANT):
    rDst = theta;
    break; 
  case (SINE_LAW):
    rDst = sin(theta);
    break; 
  case (EQUI_SOLID_ANGLE):
    rDst = sin(theta/2);
    break; 
  default:
#ifdef BIAS_DEBUG
    BIASERR("Invalid projection type!");
#endif
    return -1;
  }

  Dest.Set(rDst * cos(phi), rDst * sin(phi));
  //cout << "ProjectCoord Src:" << rDst << "," << Dest << endl;
  Dest = kMatrix * Dest;
  //cout << "ProjectCoord Src:" << rDst << "," << Dest << endl;

  return 0;
}

int SphericalUndistortion::ProjectCoordBack(const int projectionType, 
                                        const KMatrix& kMatrix,
                                        const HomgPoint2D& Source,
                                        HomgPoint2D &Dest)
{
  
  if (!isInitialized_) {
#ifdef BIAS_DEBUG
    BIASERR("Sphere Projections was not initialized!");
#endif
    return -1;
  }
  
  HomgPoint2D SourceK;
  SourceK = kMatrix.Invert()  * Source;
  
  //cout << "K" << K <<endl;
  //cout << "Source" << Source << "," << SourceK << endl;
  // distance of dst-point to center
  
  double rDst = sqrt(SourceK[0]*SourceK[0]+SourceK[1]*SourceK[1]);
  //cout << "rDst" << rDst << "," << SourceK << endl;
  // phi: angle between positive x-axis and dst-point
  double phi, theta;
  phi = atan2(SourceK[1],SourceK[0]);
  
  // theta: calibrated angle between optical axis and source image point
  switch (projectionType) {
  case (PERSPECTIVE):
    theta = atan(rDst);
    break; 
  case (STEREOGRAPHIC):
    theta = 2.0*atan(rDst);
    break; 
  case (EQUIDISTANT):
    theta = rDst;
    break; 
  case (SINE_LAW):
    if (rDst>1) {
      theta = M_PI;
    } else { 
      theta = asin(rDst);
    }
    break; 
  case (EQUI_SOLID_ANGLE):
    if (rDst>1) {
      theta = M_PI;
    } else { 
      theta = 2*asin(rDst);
    }
    break; 
  default:
#ifdef BIAS_DEBUG
    BIASERR("Invalid projection type!");
#endif
    return -1;
  }

  TransfCoordSphere2Image(theta, phi, Dest[0], Dest[1]);
  Dest[2]=1;
  //cout << Source << "," << Dest << "," << theta << "," << phi << endl;
  return 0;
  
}

template <class StorageType>
int SphericalUndistortion::DrawCircles(Image<StorageType>& src, 
                                       const double& DegreeStep) {

  if (!isInitialized_) {
#ifdef BIAS_DEBUG
    BIASERR("Sphere Projections was not initialized!");
#endif
    return -1;
  }

  StorageType* Color;
  Color = new StorageType[src.GetChannelCount()];
  Color[0]=127;
  for (unsigned int j=1; j<src.GetChannelCount(); j++) {
    Color[j]=0;
  }
  for (double theta=DegreeStep/180.0*M_PI; theta<=MaxCamAngle_; theta+=DegreeStep/180.0*M_PI) {
    double CalMaxCamAngle;
    double CalTheta;
    // calculate calibrated angles
    interpolatorAngle_.Spline(CalMaxCamAngle, MaxCamAngle_,4);  
    interpolatorAngle_.Spline(CalTheta, theta,4);  
    // apply calculate calibrated radius from calibrated angles
    double rSrc = radius_ * CalTheta/CalMaxCamAngle;
    
    if (ImageDraw<StorageType>::CircleCenter(src, (int)CenterX_, (int)CenterY_, (int)rSrc, Color)!=0){
      delete [] Color;
      return -1;
    } 
  }
  delete [] Color;
  return 0;  
}


int SphericalUndistortion::TransfCoordImage2Sphere (const HomgPoint2D& Source, 
                                                double &theta, 
                                                double &phi) {
  double SourceX, SourceY;
  SourceX = Source[0]/Source[2];
  SourceY = Source[1]/Source[2];
  theta = phi = 0;

  return TransfCoordImage2Sphere (SourceX, SourceY, theta, phi);

}


int SphericalUndistortion::TransfCoordImage2Sphere(const double& SourceX, 
                                               const double& SourceY,
                                               double &theta, 
                                               double &phi) {
  if (!isInitialized_) {
#ifdef BIAS_DEBUG
    BIASERR("Sphere Projections was not initialized!");
#endif
    return -1;
  }

  theta = phi = 0;

  double rSrc = sqrt((SourceX-CenterX_)*(SourceX-CenterX_)+(SourceY-CenterY_)*(SourceY-CenterY_));

  if (rSrc>radius_) {
#ifdef BIAS_DEBUG
    //BIASERR("point not in circle image!");
#endif
    return -1;
  }
  //cout << CenterX_ << " " <<  CenterY_ << " " 
  //<<  MaxCamAngle_ << " " << radius_ << endl;
  phi = atan2(SourceY-CenterY_,SourceX-CenterX_);
  
  double CalMaxCamAngle=0;
  // calculate calibrated maximum angle
  interpolatorAngle_.Spline(CalMaxCamAngle, MaxCamAngle_, 4);  
  // calculate calibrated maximum radius
  double RadiusCalMax = MaxCamAngle_/CalMaxCamAngle*radius_;
  // calculate calibrated theta and inverse
  double CalTheta = rSrc/RadiusCalMax*MaxCamAngle_;
  interpolatorAngleBack_.Spline(theta, CalTheta,4); 

  return 0;
}

int SphericalUndistortion::TransfCoordRotateSphere2Image(double theta, 
                                                     double phi,
                                                     const double& viewCenterX, 
                                                     const double& viewCenterY,
                                                     HomgPoint2D &Source)
{
  double sx, sy;
  sx=sy=0;
  if (TransfCoordRotateSphere2Image(theta, phi, 
                                    viewCenterX, viewCenterY,
                                    sx, sy)<0) {
    return -1;
  } else {
    Source[0] = sx;
    Source[1] = sy;
    Source[2] = 1;
    return 0;
  }
}

int 
SphericalUndistortion::TransfCoordRotateSphere2Image(double theta, 
                                                     double phi,
                                                     const double& viewCenterX,
                                                     const double& viewCenterY,
                                                     double &SourceX, 
                                                     double &SourceY)
{
  if (!isInitialized_) {
#ifdef BIAS_DEBUG
    BIASERR("Sphere Projections was not initialized!");
#endif
    return -1;
  }
  
  SourceX = SourceY = 0;
  double viewTheta, viewPhi;
  TransfCoordImage2Sphere(viewCenterX, viewCenterY, viewTheta, viewPhi);
  // avoid z-rotation by changing the rotation axes
  Vector3<double> vecPoint(1.0,phi,theta);
  Vector3<double> vecView(1.0,viewPhi,viewTheta);
  //Vector3<double> vecCenter(0,0,1);
  Vector3<double> vecAxis(cos(viewPhi+M_PI/2),sin(viewPhi+M_PI/2),0);
  Vector3<double> vecRot(0,0,0);
  vecPoint = vecPoint.CoordSphereToEuclidian();
  vecView = vecView.CoordSphereToEuclidian();
  /*
  // With 1 rotation and moving axis
  RMatrix rMat;
	rMat.Set(vecAxis, viewTheta);
  double x,y,z;
  rMat.GetRotationAnglesZYX(x,y,z);
  vecRot = rMat*vecPoint;
  //cout << x << "," << y << "," << z << endl;
  */

  //With Quaternions
  Quaternion<double> rot;
  rot.SetValueAsAxisRad (vecAxis, viewTheta);
  rot.MultVec(vecPoint, vecRot);
  
  /* 
  //With 2 rotations alpha, beta (wrong rotation!!)
  // viewAlpha: angle between x-z-Plane and vecView
  double viewAlpha = -1 * atan2(vecView[1],vecView[2]);
  // viewBeta: angle between y-z-Plane and vecView
  double viewBeta = atan2(vecView[0],vecView[2]); 
  RMatrix rMat;
  rMat.SetZYX(viewAlpha,viewBeta,0);
  vecRot = rMat*vecPoint;
  */

  vecRot = vecRot.CoordEuclidianToSphere();
  phi = vecRot[1];
  theta = vecRot[2];

  if (phi>M_PI) phi-=2*M_PI;

  double CalMaxCamAngle=0;
  double CalTheta=0;
  // calculate calibrated angles
  interpolatorAngle_.Spline(CalMaxCamAngle, MaxCamAngle_,4);  
  interpolatorAngle_.Spline(CalTheta, theta,4);  
  // apply calculate calibrated radius from calibrated angles
  double rSrc = radius_ * CalTheta/CalMaxCamAngle;
 
  SourceX = rSrc * cos(phi) + CenterX_;
  SourceY = rSrc * sin(phi) + CenterY_;

  return 0;
}


int SphericalUndistortion::TransfCoordSphere2Image(const double& theta, 
                                                   const double& phi, 
                                                   HomgPoint2D &Source)
{
  double sx, sy;
  sx=sy=0;
  if (TransfCoordSphere2Image(theta, phi, sx, sy)<0) {
    return -1;
  } else {
    Source[0] = sx;
    Source[1] = sy;
    Source[2] = 1;
    return 0;
  }
}


int SphericalUndistortion::TransfCoordSphere2Image(const double &theta,
                                                   const double &phi, 
                                                   double &SourceX, 
                                                   double &SourceY) {
  
  if (!isInitialized_) {
#ifdef BIAS_DEBUG
    BIASERR("Sphere Projections was not initialized!");
#endif
    return -1;
  }
  
  SourceX = SourceY = 0;
  
  if (theta<0 || theta>MaxCamAngle_) {
#ifdef BIAS_DEBUG
    BIASERR("Angle theta out of range!");
#endif
    return -1;
  }

  if (phi<-M_PI || phi>M_PI) {
#ifdef BIAS_DEBUG
    BIASERR("Angle phi out of range!");
#endif
    return -1;
  }

  double CalMaxCamAngle=0;
  double CalTheta=0;
  // calculate calibrated angles
  interpolatorAngle_.Spline(CalMaxCamAngle, MaxCamAngle_,4);  
  interpolatorAngle_.Spline(CalTheta, theta,4);  
  // apply calculate calibrated radius from calibrated angles
  double rSrc = radius_ * CalTheta/CalMaxCamAngle;
 
  SourceX = rSrc * cos(phi) + CenterX_;
  SourceY = rSrc * sin(phi) + CenterY_;

  return 0;
}

int SphericalUndistortion::TransfCoordSphere2CalibImage(const double &theta, 
                                                        const double &phi,
                                                        HomgPoint2D &Source)
  const {
  double sx, sy;
  sx=sy=0;
  if (TransfCoordSphere2CalibImage(theta, phi, sx, sy)<0) {
    return -1;
  } else {
    Source[0] = sx;
    Source[1] = sy;
    Source[2] = 1;
    return 0;
  }
}

int SphericalUndistortion::EnforceNormalRange(double& theta, 
                                              double& phi) const {
  // theta and phi are periodic in 2*M_PI, if they are out of a suitable
  // range, try to find best equivalent angle by adding 2*l*M_PI

  if (theta<0.0) {
    const int count = (int) ((double)theta / (M_PI*(-2.0)));
    theta += ((double)count+1.0) * M_PI*2.0;
    if (theta>MaxCamAngle_) {
      BIASERR("Angle theta out of range!"<<theta);
      return -1;
    } 
    //cout <<"(1) fixed theta="<<theta<<endl;
  } else if (theta>MaxCamAngle_) {
    const int count = (int) ((double)theta / (M_PI*(2.0)));
    theta -= ((double)count+1.0) * M_PI*2.0;
    if (theta<0.0) {
      BIASERR("Angle theta out of range: "<<theta);
      return -2;
    } 
    //cout <<"(2) fixed theta="<<theta<<endl;
  }
 
  if (phi<-M_PI) {
    const int count = (int) ((double)phi / (M_PI*(-2.0)));
    phi += ((double)count+1.0) * M_PI*2.0;
    if (phi>M_PI) {
      BIASERR("Angle phi out of range:"<<phi);
      return -3;
    } 
    //cout <<"(1) fixed phi="<<phi<<endl;
  } else if (phi>M_PI) {
    const int count = (int)((double)phi / (M_PI*(2.0)));
    phi -= ((double)count+1.0) * M_PI*2.0;
    if (phi<-M_PI) {
      BIASERR("Angle phi out of range: "<<phi);
      return -4;
    } 
    //cout <<"(2) fixed phi="<<phi<<endl;
  }
  return 0;
}

int SphericalUndistortion::TransfCoordSphere2CalibImage(const double& theta, 
                                                        const double& phi,
                                                        double &SourceX, 
                                                        double &SourceY) const{
  
  if (!isInitialized_) {
#ifdef BIAS_DEBUG
    BIASERR("Sphere Projections was not initialized!");
#endif
    return -1;
  }
  
  SourceX = SourceY = 0;
  
  if (theta<0 || theta>MaxCamAngle_) {
#ifdef BIAS_DEBUG
    //BIASERR("Angle theta out of range!");
#endif
    return -1;
  }

  if (phi<-M_PI || phi>M_PI) {
#ifdef BIAS_DEBUG
    //BIASERR("Angle phi out of range!");
#endif
    return -1;
  }

  double CalMaxCamAngle=0;
  double CalTheta=0;
  // apply radius from calibrated angles
  double rSrc = radius_ * CalTheta/CalMaxCamAngle;

  SourceX = rSrc * cos(phi) + CenterX_;
  SourceY = rSrc * sin(phi) + CenterY_;

  return 0;

}


// JW
#ifdef WIN32
# ifdef max
#   undef max
#  endif // max
#endif //WIN32


template <class StorageType>
int SphericalUndistortion::CorrectVignette(Image<StorageType>& src) {

  if (!isInitialized_) {
#ifdef BIAS_DEBUG
    BIASERR("Sphere Projections was not initialized!");
#endif
    return -1;
  }
  if (IlluCorrX_.size()<2 || IlluCorrY_.size()<2) return 0;
  
  StorageType** Data=src.GetImageDataArray();

  const StorageType maxVal = (StorageType) ::std::numeric_limits<StorageType>::max(); // JW
  double illuFac=0;
  double theta=0, phi=0;
  for (unsigned int y=0; y<src.GetHeight(); y++) {
    for (unsigned int x=0; x<src.GetWidth(); x++) {
      HomgPoint2D hp(x,y);
      TransfCoordImage2Sphere(hp, theta, phi);
      for (unsigned int j=0; j<src.GetChannelCount(); j++) {
        interpolatorIllu_.Spline(illuFac, theta,4);
		// clip overflow range: JW
		// JW TODO: liek this: if ((double)Data[y][x*src.GetChannelCount()+j]) / (double)maxVal >= illuFac ) {
		if (Data[y][x*src.GetChannelCount()+j]/illuFac > maxVal) {
          Data[y][x*src.GetChannelCount()+j] = maxVal;
        } else {
          Data[y][x*src.GetChannelCount()+j] = 
            StorageType((float)Data[y][x*src.GetChannelCount()+j]/illuFac);
        }
      }
    }
  }
  return 0;
}

KMatrix SphericalUndistortion::GetKMatrix() {
  
  KMatrix K;
  K.SetIdentity();

  if (!isInitialized_) {
#ifdef BIAS_DEBUG
    BIASERR("Sphere Projections was not initialized!");
#endif
    return K;
  }  
  double cx, cy;
  GetSphereCenter(cx, cy);
  double sx, sy;
  TransfCoordSphere2Image( 75.0/180.0*M_PI, 0.0, sx, sy);
  K[0][0] = (sx-cx)/tan(75.0/180.0*M_PI);
  K[1][1] = (sx-cx)/tan(75.0/180.0*M_PI);
  K[0][1] = 0; //skew=0!
  K[0][2] = CenterX_;
  K[1][2] = CenterY_;
 
  return K;
} 


template <class StorageType>
int SphericalUndistortion::CalcCentGrav_(const Image<StorageType> &src, 
                                         double &centerX, 
                                         double &centerY) const{

  //unsigned int thresh = 40;
  StorageType thresh = StorageType(40);

  double centX = 0;
  double centY = 0;
  double count = 0;
  StorageType** Data=src.GetImageDataArray();
  for (unsigned int y=0; y<src.GetHeight(); y++) {
    for (unsigned int x=0; x<src.GetWidth(); x++) {
      for (unsigned int j=0; j<src.GetChannelCount(); j++) {
        if (Data[y][x*src.GetChannelCount()+j]>thresh) {
          centX += x;
          centY += y;
          count++;
        }
      }
    }
  }
  
  centerX = centX/count;
  centerY = centY/count;
  
  return 0;
}


// Calculate difference of (points inside circle - points outside circle)
// given is circle center and radius.
// The function also returns the "center og gravity of the gradients"
// indicating a vector from the center do the region of best match
// to the highest gradient.

template <class StorageType>
int SphericalUndistortion::CalcCircularGradient_(const Image<StorageType>& src,
                                                 double CenterX, 
                                                 double CenterY, 
                                                 double Radius,  
                                                 double &CentGravX,
                                                 double &CentGravY, 
                                                 double &curGradSum) const {

  int maxX = src.GetWidth()-1;
  int maxY = src.GetHeight()-1;

  double xOut, yOut, xIn, yIn;
  curGradSum = 0;
  double In, Out;
  double CentGravDiv=0;
  CentGravX = CentGravY = 0; 
  double Thresh=15;

  for (double phi=0; phi<2*M_PI; phi+=M_PI/100) {

    xOut = (Radius+1) * cos(phi) + CenterX;
    yOut = (Radius+1) * sin(phi) + CenterY;
    xIn  = (Radius-3) * cos(phi) + CenterX;
    yIn  = (Radius-3) * sin(phi) + CenterY;

    if (xOut<1 || xOut>maxX || yOut<1 || yOut>maxY) {
      return -1;
    } else {
      for (unsigned int j=0; j<src.GetChannelCount(); j++) { 
        In  = src.BilinearInterpolation(xIn,yIn,j);
        Out = src.BilinearInterpolation(xOut,yOut,j);
        curGradSum += In-1.1*Out;
        if (In>Out) {
          CentGravX   += (In-Out)>Thresh ? xOut : 0;
          CentGravY   += (In-Out)>Thresh ? yOut : 0;
          CentGravDiv += (In-Out)>Thresh ? 1 : 0;
       }
      }
    }
  }

  if (CentGravDiv>0) {
    CentGravX /= CentGravDiv;
    CentGravY /= CentGravDiv;
  } else {
    CentGravX = CenterX-1;
    CentGravY = CenterY-1;
  }
    
  return 0;
} 

template <class StorageType>
int SphericalUndistortion::FitCircle(const Image<StorageType>& src, 
                                     double &CenterX, 
                                     double &CenterY, double &Radius)  {
  
  if (CalcCentGrav_(src, CenterX, CenterY)<0) {
    return -1;
  }

  double curRad=0;
  double curCentX=CenterX;
  double curCentY=CenterY;
  double curGradSum=0;
  double curCentGravCircGradX=0;
  double curCentGravCircGradY=0;

  double bestRad=0;
  double bestCentX=CenterX;
  double bestCentY=CenterY;
  double bestGradSum=-100000;
  double bestCentGravCircGradX=0;
  double bestCentGravCircGradY=0;

  double oldRad=0;
  double oldCentX=CenterX;
  double oldCentY=CenterY;
  double oldGradSum=-100000;

  double curAdd=1;

  bool gradSumImproved = true; 
  while (gradSumImproved) {

    oldGradSum = bestGradSum;

    bool borderReached=false;
    curRad = bestRad<30 ? 10 : bestRad-20;    
    curCentX = bestCentX;
    curCentY = bestCentY;

    while (!borderReached) {

      // enlarge a circle in small steps and find the circle
      // size with the highest gradient 
      if (CalcCircularGradient_(src, curCentX, curCentY, curRad, 
          curCentGravCircGradX, curCentGravCircGradY, curGradSum)<0 || 
          curRad*2>src.GetHeight() ) {
        curGradSum=-100000;
        borderReached=true;
      }

      if (curGradSum>bestGradSum) {
        bestGradSum =curGradSum;
        bestRad=curRad;
        bestCentX=curCentX;
        bestCentY=curCentY;
        bestCentGravCircGradX=curCentGravCircGradX;
        bestCentGravCircGradY=curCentGravCircGradY;
      } 
            
      //cout <<"  X"<< curCentX << "  Y"<< curCentY 
      //     <<"  R"<< curRad <<"  Gs"<< curGradSum << endl;
      curRad += curAdd;
    }
    //cout << endl << "Result size" << endl;
    //cout <<"  X"<< bestCentX << "  Y"<<  bestCentY
    //     <<"  R"<< bestRad <<"  Gs"<< bestGradSum
    //     <<"  CGCx"<< bestCentGravCircGradX 
    //     <<"  CGCy"<< bestCentGravCircGradY
    //     << endl << endl;
  
    // shifting and enlarging the existing circle will correct the 
    // circle center and size in one loop

    oldRad=bestRad;
    oldCentX=bestCentX;
    oldCentY=bestCentY;
    borderReached=false;
    curRad   = oldRad-20;
    double gradSquare = 
      (bestCentY-bestCentGravCircGradY)/(bestCentX-bestCentGravCircGradX) * 
      (bestCentY-bestCentGravCircGradY)/(bestCentX-bestCentGravCircGradX);

    while (curRad<oldRad+20 && !borderReached) {      

      curCentX = oldCentX + sqrt(gradSquare/(1+gradSquare))*(oldRad-curRad);
      curCentY = oldCentY + sqrt(1/(1+gradSquare))*(oldRad-curRad);

      if (CalcCircularGradient_(src, curCentX, curCentY, curRad, 
          curCentGravCircGradX, curCentGravCircGradY, curGradSum)<0 || 
          curRad*2>src.GetHeight() ) {
        curGradSum=-100000;
        borderReached=true;
      }

      if (curGradSum>bestGradSum) {
        bestGradSum =curGradSum;
        bestRad=curRad;
        bestCentX=curCentX;
        bestCentY=curCentY;
        bestCentGravCircGradX=curCentGravCircGradX;
        bestCentGravCircGradY=curCentGravCircGradY;
      } 
      //cout <<"  X"<< curCentX << "  Y"<< curCentY 
      //     <<"  R"<< curRad <<"  Gs"<< curGradSum << endl;
      
      curRad+=curAdd/2;
    } 
 
    //cout << endl << "Result Center Shift" << endl;
    //cout <<"  X"<< bestCentX << "  Y"<<  bestCentY
    //     <<"  R"<< bestRad <<"  Gs"<< bestGradSum
    //     <<"  CGCx"<< bestCentGravCircGradX 
    //     <<"  CGCy"<< bestCentGravCircGradY
    //     << endl << endl;
  
    oldRad=bestRad;
    oldCentX=bestCentX;
    oldCentY=bestCentY;
    borderReached=false;
    curRad   = oldRad-20;

    while (curRad<oldRad+20 && !borderReached) {      

      curCentX = oldCentX - sqrt(1/(1+gradSquare))*(oldRad-curRad);
      curCentY = oldCentY + sqrt(gradSquare/(1+gradSquare))*(oldRad-curRad);

      if (CalcCircularGradient_(src, curCentX, curCentY, curRad, 
          curCentGravCircGradX, curCentGravCircGradY, curGradSum)<0 || 
          curRad*2>src.GetHeight() ) {
        curGradSum=-100000;
        borderReached=true;
      }

      if (curGradSum>bestGradSum) {
        bestGradSum =curGradSum;
        bestRad=curRad;
        bestCentX=curCentX;
        bestCentY=curCentY;
        bestCentGravCircGradX=curCentGravCircGradX;
        bestCentGravCircGradY=curCentGravCircGradY;
      } 
      //cout <<"  X"<< curCentX << "  Y"<< curCentY 
      //     <<"  R"<< curRad <<"  Gs"<< curGradSum << endl;
      
      curRad+=curAdd/2;
    }    
    //cout << endl << "Result Center Shift Orthogonal" << endl;
    //cout <<"  X"<< bestCentX << "  Y"<<  bestCentY
    //     <<"  R"<< bestRad <<"  Gs"<< bestGradSum 
    //     <<"  CGCx"<< bestCentGravCircGradX 
    //     <<"  CGCy"<< bestCentGravCircGradY
    //     << endl << endl;
    
   if (oldGradSum >= bestGradSum) {
      gradSumImproved=false;
    }
    curAdd = 0.5;

  }

  CenterX = bestCentX;
  CenterY = bestCentY;
  Radius = bestRad;
 
  return 0;
}
