/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003, 2004    (see file CONTACTS for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef __TrackerGaussianWeightedKLT__hh__
#define __TrackerGaussianWeightedKLT__hh__

#include <Utils/Param.hh>
#include "TrackerSimpleKLT.hh"


namespace BIAS {
  /** @class TrackerGaussianWeightedKLT
      @ingroup g_geometry
      @brief 

      @author woelk 09/2004 */

  
  template <class StorageType>
  class DLLEXPORTAPI TrackerGaussianWeightedKLT 
    : public TrackerSimpleKLT<StorageType>
  {
  public :
    TrackerGaussianWeightedKLT();


    TrackerGaussianWeightedKLT(Param& para, const std::string& postfix="");

    TrackerGaussianWeightedKLT(Param& para, const int grp, 
                               const std::string& postfix);

    ~TrackerGaussianWeightedKLT();
    
    virtual void Init(Image<StorageType>& im1, Image<StorageType>& im2, 
                      Image<StorageType>& gradx1, Image<StorageType>& grady1, 
                      Image<StorageType>& gradx2, Image<StorageType>& grady2);
    
    /** all homogenous points must be homogenized
        @returns  0 on success (error< maxerror)
                 -1 when no spatial structure is present
                 -2 when point slid out of image
                 -3 when maxiter is reached an error is above maxerror
                 -4 when a point lies outside of valid region in images
                  1 when error increased from last iteration */
    virtual int Track(HomgPoint2D& p1, HomgPoint2D& p2, HomgPoint2D& result, 
                      KLT_TYPE& error, int &iter, KLT_TYPE& residuum);

    /** just the function prototype */
    virtual void Track(std::vector<HomgPoint2D>& p1, 
                       std::vector<HomgPoint2D>& p2, 
                       std::vector<HomgPoint2D>& p2tracked, 
                       std::vector<KLT_TYPE>& error, 
                       std::vector<int>& numiter,
                       std::vector<KLT_TYPE>& residui, 
                       std::vector<int>& res);

    /////////////////////////////////////////////////////////////////////////
    //    basic functions, needed, when images are no longer present
    /////////////////////////////////////////////////////////////////////////

    /** see TrackerBase::Init for description */
    virtual 
    void Init(StorageType **ida1, StorageType **ida2, StorageType **gradx1, 
              StorageType **grady1, StorageType **gradx2, 
              StorageType **grady2, unsigned int width, 
	      unsigned int height,unsigned gradient_half_mask);

    

    /** @brief simple gradient decent algorithm after 
        Lucas and Kanade. Everey pixel in support window is weighted 
        gaussian according to its position relative to window center */
    virtual int Track(KLT_TYPE p1[2], KLT_TYPE p2[2], KLT_TYPE result[2],
                      KLT_TYPE& error, int &iter, KLT_TYPE& residuum);
    

    
  protected:
    Matrix<KLT_TYPE> _gauss;
    
    // does the actual gradient descent
    int Track_(KLT_TYPE p1[2], KLT_TYPE p2[2], KLT_TYPE result[2], 
               KLT_TYPE& error, int &iter, KLT_TYPE& residuum);

    // recalculates the _gauss kernel
    void CalcGauss_();

  }; // class


} // namespace


#endif // __TrackerGaussianWeightedKLT__hh__
