/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003, 2004    (see file CONTACTS for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "TrackerSimpleKLT.hh"

  //#define TIMING
#ifdef TIMING
#include <Base/Debug/TimeMeasure.hh>
#endif

using namespace BIAS;
using namespace std;


//////////////////////////////////////////////////////////////////////////
//                 instantiation
//////////////////////////////////////////////////////////////////////////

template class TrackerSimpleKLT<unsigned char>; 
template class TrackerSimpleKLT<float>; 

// fill in instances as required
#ifdef BUILD_IMAGE_INT
template class TrackerSimpleKLT<int>; 
#endif
#ifdef BUILD_IMAGE_CHAR
template class TrackerSimpleKLT<char>; 
#endif
#ifdef BUILD_IMAGE_SHORT
template class TrackerSimpleKLT<short>; 
#endif
#ifdef BUILD_IMAGE_USHORT
template class TrackerSimpleKLT<unsigned short>; 
#endif
#ifdef BUILD_IMAGE_UINT
template class TrackerSimpleKLT<unsigned int>; 
#endif
#ifdef BUILD_IMAGE_DOUBLE
#endif

//////////////////////////////////////////////////////////////////////////
//                 macro(s) for fast bilinear interpolation
//////////////////////////////////////////////////////////////////////////

//static unsigned rf, rc, cf, cc, r, c, x_floor, y_floor,x_floor_end,y_floor_end;
//static KLT_TYPE ul, ur, ll, lr, brdx, brdy, brdxy;

// for MyBilinearRegion
/*
#define MyBilinearRegionUC(ida, gx, gy, x, y, hws, rida, rgx, rgy) \
  x_floor = (unsigned int)floor(x);\
  y_floor = (unsigned int)floor(y);\
  x_floor_end = x_floor+hws+1;\
  y_floor_end = y_floor+hws+1;\
  brdy = (KLT_TYPE)(y - y_floor);\
  brdx = (KLT_TYPE)(x - x_floor);\
  brdxy = brdx*brdy;\
  for (rf=y_floor-hws, rc=rf+1, r=0; rf<y_floor_end; rf++, rc++, r++){\
    for (cf=x_floor-hws, cc=cf+1, c=0; cf<x_floor_end; cf++, cc++, c++){\
      ul = (KLT_TYPE)ida[rf][cf];\
      ur = (KLT_TYPE)ida[rf][cc];\
      ll = (KLT_TYPE)ida[rc][cf];\
      lr = (KLT_TYPE)ida[rc][cc];\
      rida[r][c]=(ul + brdy*(ll - ul) + brdx*(ur - ul) + \
                  brdxy*(ul + lr - ll - ur));\
      ul = (KLT_TYPE)(gx[rf][cf])-(KLT_TYPE)(GRAD_UCHAR_ZERO);\
      ur = (KLT_TYPE)(gx[rf][cc])-(KLT_TYPE)(GRAD_UCHAR_ZERO);\
      ll = (KLT_TYPE)(gx[rc][cf])-(KLT_TYPE)(GRAD_UCHAR_ZERO);\
      lr = (KLT_TYPE)(gx[rc][cc])-(KLT_TYPE)(GRAD_UCHAR_ZERO);\
      rgx[r][c]=(ul + brdy*(ll - ul) + brdx*(ur - ul) + \
                 brdxy*(ul + lr - ll - ur));\
      ul = (KLT_TYPE)(gy[rf][cf])-(KLT_TYPE)(GRAD_UCHAR_ZERO);\
      ur = (KLT_TYPE)(gy[rf][cc])-(KLT_TYPE)(GRAD_UCHAR_ZERO);\
      ll = (KLT_TYPE)(gy[rc][cf])-(KLT_TYPE)(GRAD_UCHAR_ZERO);\
      lr = (KLT_TYPE)(gy[rc][cc])-(KLT_TYPE)(GRAD_UCHAR_ZERO);\
      rgy[r][c]=(ul + brdy*(ll - ul) + brdx*(ur - ul) + \
                 brdxy*(ul + lr - ll - ur));\
       }\
  }

#define MyBilinearRegion(ida, gx, gy, x, y, hws, rida, rgx, rgy) \
  x_floor = (unsigned int)floor(x);\
  y_floor = (unsigned int)floor(y);\
  x_floor_end = x_floor+hws+1;\
  y_floor_end = y_floor+hws+1;\
  brdy = (KLT_TYPE)(y - y_floor);\
  brdx = (KLT_TYPE)(x - x_floor);\
  brdxy = brdx*brdy;\
  for (rf=y_floor-hws, rc=rf+1, r=0; rf<y_floor_end; rf++, rc++, r++){\
    for (cf=x_floor-hws, cc=cf+1, c=0; cf<x_floor_end; cf++, cc++, c++){\
      ul = (KLT_TYPE)ida[rf][cf];\
      ur = (KLT_TYPE)ida[rf][cc];\
      ll = (KLT_TYPE)ida[rc][cf];\
      lr = (KLT_TYPE)ida[rc][cc];\
      rida[r][c]=(ul + brdy*(ll - ul) + brdx*(ur - ul) + \
                  brdxy*(ul + lr - ll - ur));\
      ul = (KLT_TYPE)(gx[rf][cf]);\
      ur = (KLT_TYPE)(gx[rf][cc]);\
      ll = (KLT_TYPE)(gx[rc][cf]);\
      lr = (KLT_TYPE)(gx[rc][cc]);\
      rgx[r][c]=(ul + brdy*(ll - ul) + brdx*(ur - ul) + \
                 brdxy*(ul + lr - ll - ur));\
      ul = (KLT_TYPE)(gy[rf][cf]);\
      ur = (KLT_TYPE)(gy[rf][cc]);\
      ll = (KLT_TYPE)(gy[rc][cf]);\
      lr = (KLT_TYPE)(gy[rc][cc]);\
      rgy[r][c]=(ul + brdy*(ll - ul) + brdx*(ur - ul) + \
                 brdxy*(ul + lr - ll - ur));\
       }\
  }
*/




//////////////////////////////////////////////////////////////////////////
//                 implementation
//////////////////////////////////////////////////////////////////////////


template <class StorageType>
TrackerSimpleKLT<StorageType>::
TrackerSimpleKLT() 
  : TrackerBase<StorageType>()
{}

template <class StorageType>
TrackerSimpleKLT<StorageType>::
TrackerSimpleKLT(Param& para, const std::string& postfix) 
  : TrackerBase<StorageType>(para, postfix)
{
  _LastHalfWinSize=_WinSize=0;
}

template <class StorageType>
TrackerSimpleKLT<StorageType>::
TrackerSimpleKLT(Param& para, const int grp, const std::string& postfix) 
  : TrackerBase<StorageType>(para, grp, postfix)
{  
  _LastHalfWinSize=_WinSize=0;
}

template <class StorageType>
TrackerSimpleKLT<StorageType>::~TrackerSimpleKLT()
{}

template <class StorageType>
void TrackerSimpleKLT<StorageType>::
Init(Image<StorageType>& im1, Image<StorageType>& im2, 
     Image<StorageType>& gradx1, Image<StorageType>& grady1,
     Image<StorageType>& gradx2, Image<StorageType>& grady2)
{
  if (*this->_HalfWinSize != _LastHalfWinSize){
    _Resize(*this->_HalfWinSize);
  }
  TrackerBase<StorageType>::Init(im1, im2, gradx1, grady1, gradx2, grady2);
}



template <class StorageType>
void TrackerSimpleKLT<StorageType>::
Track(std::vector<HomgPoint2D>& p1, std::vector<HomgPoint2D>& p2, 
      std::vector<HomgPoint2D>& p2tracked, std::vector<KLT_TYPE>& error, 
      std::vector<int>& numiter, std::vector<KLT_TYPE>& residui, 
      std::vector<int>& res)
{
#ifdef TIMING
  TimeMeasure timer;
  timer.Start();
#endif
  const unsigned num=p1.size();
  BIASASSERT(num==p2.size() && num==p2tracked.size() && num==error.size());
  BIASASSERT(num==numiter.size() && num==res.size());

  for (unsigned i=0; i<num; i++){
    if (p1[i][2]==1.0){
#ifdef BIAS_DEBUG
      BIASASSERT(p2[i].IsHomogenized());
      BIASCDOUT(D_TRACKERB_INPUT, "attemting to track "<<setw(4)<<i<<"  "
                <<p1[i]<<" -> "<<p2[i]);
#endif    
      res[i]=Track_(p1[i].GetData(), p2[i].GetData(), p2tracked[i].GetData(), 
                    error[i], numiter[i], residui[i]);
      
    } else {
      res[i] = TRACKER_IGNOREPOINT;
    }
  }
#ifdef TIMING
  timer.Stop();
  cerr << "TrackerSimpleKLT tracking took "<<timer.GetRealTime()<<" us\n";
#endif
}

template <class StorageType>
int TrackerSimpleKLT<StorageType>::
Track(HomgPoint2D& p1, HomgPoint2D& p2, HomgPoint2D& result, 
      KLT_TYPE& error, int &iter, KLT_TYPE& residuum)
{
#ifdef BIAS_DEBUG
  BIASASSERT(p1.IsHomogenized() && p2.IsHomogenized());
#endif  
  return Track_(p1.GetData(), p2.GetData(), result.GetData(), error, iter,
                residuum);
}


template <class StorageType>
void TrackerSimpleKLT<StorageType>::
Init(StorageType **ida1, StorageType **ida2, StorageType **gradx1, 
     StorageType **grady1, StorageType **gradx2, StorageType **grady2, 
     unsigned int width, unsigned int height, unsigned gradient_half_mask)
{
  if (*this->_HalfWinSize != _LastHalfWinSize){
    _Resize(*this->_HalfWinSize);
  }
  TrackerBase<StorageType>::Init(ida1, ida2, gradx1, grady1, gradx2, grady2, 
                                 width, height, gradient_half_mask);
}


template <class StorageType>
int TrackerSimpleKLT<StorageType>::
Track(KLT_TYPE p1[2], KLT_TYPE p2[2], KLT_TYPE result[2], 
      KLT_TYPE& error, int &iter, KLT_TYPE& residuum)
{
  return Track_(p1, p2, result, error, iter, residuum);
}


template <class StorageType>
int TrackerSimpleKLT<StorageType>::
Track_(KLT_TYPE p1[2], KLT_TYPE p2[2], KLT_TYPE result[2], 
       KLT_TYPE& error, int &iter, KLT_TYPE& residuum)
{
#ifdef TIMING
  TimeMeasure timer;
  timer.Start();
#endif
  if (*this->_HalfWinSize != _LastHalfWinSize){
    _Resize(*this->_HalfWinSize);
  }
  BIASCDOUT(D_TRACKERB_KLT,"trying to track p1: (" << p1[0] << ", " << p1[1] 
            << ") <--> p2: (" <<p2[0] << ", " << p2[1]<<")\n");

  int res=0;
  int rh, ch;
  
  register KLT_TYPE dI, gxx, gxy, gyy, ex, ey, dx, dy, det, idet;
  KLT_TYPE lsterror=FLT_MAX;
  const int hws=*this->_HalfWinSize;
  const int winsize=_WinSize;
  const int maxiter=*this->_MaxIterations;
  const KLT_TYPE maxerr=*this->_MaxError;
  const KLT_TYPE max_residuum=*this->_MaxResiduum;

  if (p1[0]<this->_minx || p1[0]>=this->_maxx || p1[1]<this->_miny || 
      p1[1]>=this->_maxy || 
      p2[0]<this->_minx || p2[0]>=this->_maxx || p2[1]<this->_miny || 
      p2[1]>=this->_maxy ){
    BIASCDOUT(D_TRACKERB_KLT, "points do not lie in ROI, returning -4\n");
    BIASCDOUT(D_TRACKERB_KLT, "("<<this->_minx<<", "<<this->_miny<<") <--> ("
              <<this->_maxx<<", "<<this->_maxy<<")\n");
    return TRACKER_OUTOFROI;
  } else {
    BIASCDOUT(D_TRACKERB_KLT, "ROI ("<<this->_minx<<", "<<this->_miny
              <<") <--> ("<<this->_maxx<<", "<<this->_maxy<<")\n")
  }

  iter=0;

  result[0]=p2[0];
  result[1]=p2[1];
  //  MyBilinearRegion(_ida1,_gradx1,_grady1,p1[0],p1[1],hws,_bl1,_gx1,_gy1);
  _MyBilinearRegion1(p1[0],p1[1],hws);
  BIASCDOUT(D_TRACKERB_BILINEAR, "_bl1: "<<_bl1<<"\n_gx1: "<<_gx1
            <<"\n_gy1: "<<_gy1<<endl);
#ifdef TIMING
  timer.Stop();
  cerr << "TrackerSimpleKLT _MyBilinearRegion1 took "<<timer.GetRealTime()
       <<" us = "<<timer.GetCycleCount()<<" cycles \n";
  timer.Reset();
  timer.Start();
#endif

  do {
    gxx=gxy=gyy=ex=ey=0.0;
    //    MyBilinearRegion(_ida2,_gradx2,_grady2,result[0],result[1],hws,
    //                 _bl2,_gx2,_gy2);
    _MyBilinearRegion2(result[0],result[1],hws);
    BIASCDOUT(D_TRACKERB_BILINEAR, "_bl2: "<<_bl2<<"\n_gx2: "<<_gx2
              <<"\n_gy2: "<<_gy2<<endl);
   
    // initialize G and e
    for (rh=0; rh<winsize; rh++){
      for (ch=0; ch<winsize; ch++){
        _gsx[rh][ch] = _gx1[rh][ch] + _gx2[rh][ch];
        _gsy[rh][ch] = _gy1[rh][ch] + _gy2[rh][ch];
        gxx += _gsx[rh][ch]*_gsx[rh][ch];
        gyy += _gsy[rh][ch]*_gsy[rh][ch];
        gxy += _gsx[rh][ch]*_gsy[rh][ch];
        dI = _bl1[rh][ch] - _bl2[rh][ch];
        ex += _gsx[rh][ch]*dI;
        ey += _gsy[rh][ch]*dI;
      } 
    }
    BIASCDOUT(D_TRACKERB_KLT, "iter: "<<iter<<"  gxx: "<<gxx<<" gxy: "<<gxy
              <<" gyy: "<<gyy<<" ex: "<<ex<<" ey: "<<ey<<endl);
    det = gxx*gyy-gxy*gxy;
    if (fabs(det)<1e-8) {
      BIASDOUT(D_TRACKERB_KLT, 
               "unable to invert G, det: "<<det);
      res = TRACKER_NOSTRUCTURE;
      break;
    }
    idet=1.0/det;
    dx = (gyy*ex - gxy*ey)*idet; 
    dy = (gxx*ey - gxy*ex)*idet; 

    BIASCDOUT(D_TRACKERB_KLT, "     det: "<<det<<" idet: "<<idet
              <<" dx: "<<dx<<" dy: "<<dy<<endl);
    
    //error = sqrt(dx*dx + dy*dy);
    error = (fabs(dx)>fabs(dy))?fabs(dx):fabs(dy);
    /*
    if (error>lsterror){
      BIASCDOUT(D_TRACKERB_KLT, 
                "error increased ("
                <<lsterror<<" -> "<<error
                <<"), stopping iteration and returning 1\n");
      res = TRACKER_ERRORINCREASED;
      error=lsterror;
      break;
      } */
    lsterror=error;

    result[0] += dx;
    result[1] += dy;

    BIASCDOUT(D_TRACKERB_KLT,"     disp after "<<iter<<" iteration: "
              <<result[0]-p2[0]<<", "<<result[1]-p2[1]<<" error: "<< error
              << "  ("<<result[0]<<", "<<result[1]<<")\n");

    //#define D_TRACK
#ifdef D_TRACK
    fprintf(stdout, "iter: %i\tgxx: %f\tgxy: %f\tgyy: %f\tex: %f\tey: %f\n",
            iter, gxx, gxy, gyy, ex, ey);
    fprintf(stdout, "      \tdx: %f\tdy: %f\tcoo: (%f, %f)\n", 
            dx, dy, result[0], result[1]);
#endif

    iter++;
    
    if (result[0]<this->_minx || result[0]>=this->_maxx ||
        result[1]<this->_miny || result[1]>=this->_maxy){
      BIASCDOUT(D_TRACKERB_KLT, 
                "feature slid out of image, returning -2");
      res = TRACKER_OUTOFIMAGE;
      break;
    }
    //if (iter==2) exit(-1);
  } while (error>maxerr && iter<maxiter);
    
  if (res==0 && error>maxerr && iter==maxiter) {
    res = TRACKER_TOOMANYITER;
    BIASCDOUT(D_TRACKERB_KLT,"error > "<<maxerr
              <<" and max iteration count reached: "<<iter<<" returning -3");
  }
  
  if (res==0){
    const int wsize= winsize*winsize;
    KLT_TYPE *f1=_bl1.GetData(), *f2=_bl2.GetData(), *end=f1+wsize;
    residuum=0.0;
    while (f1<end){
      residuum += fabs(*f1++ - *f2++);
    }
    residuum/=(KLT_TYPE)wsize;
    if (residuum>max_residuum){
      res = TRACKER_MEANABSDIF;
      BIASCDOUT(D_TRACKERB_KLT, "residuuum = "<<residuum<<" > "<<max_residuum
                <<" returning -5");
    }
  }

#ifndef WIN32//sunmin 10-27-04
  BIASASSERT(!isnan(error));
#endif 
#ifdef TIMING
  timer.Stop();
  cerr << "i "<<iter<<" res "<<res<<" ";
  cerr << "TrackerSimpleKLT track_ took "<<timer.GetRealTime()<<" us = "
       <<timer.GetCycleCount()<<" cycles\n";
#endif
  return res;
}

