/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003, 2004    (see file CONTACTS for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#ifndef __BIAS_CONDENSATION_HH__
#define __BIAS_CONDENSATION_HH__
#include "./Condensation.hh"
#include "Base/Image/Image.hh"
#include "Base/Math/Vector2.hh"

namespace BIAS {

/** @class CondensImg
    @brief example class for condensation with a 2-dim state vector.
    
    The observation p(Z|X) is simply an image, where the greyvalue represents
    the probabilty.<br>
    See condensImgExample.cpp also!
    @author Daniel Grest, Feb. 2004
*/

class CondensImg : public Condensation
{
public:
  /** u have to set the stateDim in the constructor! 
   */
  CondensImg() { stateDim_=2;}

  virtual void InitModelDefaults();
  
  virtual void InitPriorPositions(unsigned int nrOfInitSamples);
 
  virtual void EvaluateObservationDensities();
  
  virtual void EvaluateImportanceWeights();

  /** The area for the initial uniform distribution.
      Probably image size...
   */     
  void SetArea(Vector2<double> min, Vector2<double> max);

  /** converts to Image<float> and calls the function above */
  void SetObservation(Image<unsigned char> &osbImg);

  /* at every (rounded) sample position, the sampleImg is set
     to UCHAR_MAX */
  void GetSamplePositions(Image<unsigned char> &sampleImg);

  /** this draws the sample weights as circles
   */
  void DrawSamples(Image<unsigned char> &img, 
                   unsigned char value=255);

  /** Draws the samples with their weight as greyvalue,
      the weight is distributed to the 4 pixels around the sample postion.
      Image has to be initialized with correct size and channelcount !
  */
  void DrawPosteriorDistribution(BIAS::Image<unsigned char> &img);

  /** same as above but float precision and 
      returns the maximum value also.
  */
  double DrawPosteriorDistribution(BIAS::Image<float> &img);
  

  /** Draws the mean as a cross, which size is proportional to the variance
   */
  void DrawMean(BIAS::Image<unsigned char> &img);

protected:
  Vector<double> areaMin_, areaMax_;
  Image<unsigned char> obsImg_;

  // needed for DrawPosteriorDistribution()
  BIAS::Image<float> fImg_;

};

}

#endif
