#include <Base/Image/ImageIO.hh>

using namespace BIAS;
using namespace std;

int main(int argc, char *argv[])
{
  int border=0;
  if (argc<3){
    cerr << argv[0] <<" <image 1> <image 2> [border]\n";
    cerr << "   calculates the absolute difference between image 1 and \n"
         << "   image 2 and stores it in diff.mip. If border is given, the\n"
         << "   difference is only calculated within border from image edges\n";
    //return -1;
	return 0; // for Dart testing
  }
  if (argc>3){
    border=atoi(argv[3]);
    cerr << "using border = "<<border<<endl;
  }

  ImageBase im1, im2, diff;
  if (ImageIO::Load(argv[1], im1)!=0){
    BIASERR("error loading "<<argv[1]);
    return -1;
  }
  if (ImageIO::Load(argv[2], im2)!=0){
    BIASERR("error loading "<<argv[2]);
    return -1;
  }

  im1.SetROI(border, border, im1.GetWidth()-border, im1.GetHeight()-border);
  im2.SetROI(border, border, im2.GetWidth()-border, im2.GetHeight()-border);
  BIASASSERT(im1.GetWidth()==im2.GetWidth());
  BIASASSERT(im1.GetHeight()==im2.GetHeight());

  Image<float> fim1, fim2, fdiff;
  Image<unsigned char> ucim1, ucim2, ucdiff;
  float fmin, fmax;
  unsigned char ucmin, ucmax;

  switch (im1.GetStorageType()){
  case ImageBase::ST_unsignedchar:
    ucim1=im1;
    ucim2=im2;
    ucdiff.AbsDiff(ucim1, ucim2);
    
    if (ImageIO::Write("diff.mip", ucdiff)!=0){
      cerr << "error writing diff.mip "<<endl;
    }
    ucdiff.GetMinMaxPixelValue(ucmin, ucmax);
    cerr << "differences between "<<(int)ucmin<<" and "<<(int)ucmax<<endl;
    if (ucdiff.ScaleShiftBetween(0, 255)!=0){
      BIASERR("error scaling image");
    }
    if (ImageIO::Write("diff-scaled.mip", ucdiff)!=0){
      cerr << "error writing diff-scaled.mip "<<endl;
    }
    break;
  case ImageBase::ST_float:
    fim1=im1;
    fim2=im2;
    fdiff.AbsDiff(fim1, fim2);

    if (ImageIO::Write("diff.mip", fdiff)!=0){
      cerr << "error writing diff.mip "<<endl;
    }
    fdiff.GetMinMaxPixelValue(fmin, fmax);
    cerr << "differences between "<<fmin<<" and "<<fmax<<endl;
    break;
  default:
    cerr << "unknown storage type\n";
    break;
  }




  return 0;
}
