/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003, 2004    (see file CONTACTS for details)
Multimediale Systeme der Informationsverarbeitung
Institut fuer Informatik
Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include <Image/Camera.hh>
#include <Base/Image/ImageIO.hh>
#include <iostream>
#include <fstream>

#ifdef   WIN32
#include "Base/Common/getopt_W32.h"
#else  //not WIN32
#include <unistd.h>
#endif //WIN32

using namespace std;
using namespace BIAS;


void usage()
{
	cout <<"Usage: biasextract [options] Image ..."<<endl
		<<" Options: "<<endl
		<<"-t      : extract timestamp as [sec usec]"<<endl
		<<"-T      : extract timestamp as [msec]"<<endl
		<<"-P      : extract P-Matrix"<<endl
		<<"-u      : extract UID"<<endl
		<<"-O      : extract orientation"<<endl
		<<"-o file : write output to file"<<endl;
}


/** @class biasextract
@ingroup g_tools
@brief ???
@author evers */
int main(int argc,char *argv[]) 
{
	bool GetTimeStamp = false;
	bool GetTimeMsec = false;
	bool GetPMatrix = false;
	bool GetUID = false;
	bool GetOri = false;
	string Outfile;
	ofstream ofs;

	//int c;
	int c=1;
	while(c >0) {
		c = getopt(argc,argv,"tTPo:uO");
		if (c>0){
			switch(c)
			{
			case 't': GetTimeStamp = true;
				break;
			case 'T': 
				GetTimeStamp = true;
				GetTimeMsec = true;	
				break;
			case 'P': GetPMatrix = true;
				break;
			case 'u': GetUID = true;
				break;
			case 'O': GetOri = true;
				break;
			case 'o': Outfile = optarg;
				break;
				//     case 'h' : 
			default:
				usage();
				//exit(1);
				return 0; // for Dart testing 
			}
		}
	}


	if (argc-optind < 1) {
		cout <<"==> To few arguments."<<endl;
		usage();
		exit(1);
	}
	if (Outfile != "") {
		ofs.open(Outfile.c_str());
		if (!ofs) {
			perror(Outfile.c_str());
			exit(1);
		}
		cout <<"Writing to file: "<<Outfile<<endl;
		cout.rdbuf(ofs.rdbuf());
	}
	else {
		//     ofs = cout;
		cout <<"Writing to stdout "<<endl;
	}


	Camera<unsigned char> pic;
	for ( int i=optind; i<argc;i++){
		//     ImageIO::Read(argv[i],pic);
		ImageIO::Load(argv[i],pic);
		pic.ParseMetaData();
		cout <<argv[i]<<"\t";
		if (GetUID)   cout<<pic.GetUID()<<"  ";  
		if (GetTimeStamp) {
			long sec,usec;
			pic.GetTime(sec,usec);
			if (GetTimeMsec) cout<< (sec % 1000) * 1000 + (usec / 1000);
			else cout<<sec<<"  "<<usec;
		}
		if (GetOri) {
			float y,p,r;
			pic.GetOrientation(y,p,r);
			cout<<y<<"   "<<p<<"   "<<r;
		}


		cout<<endl;
	}

	if (Outfile != "")  
		ofs.close();

	return 0;
}
